#include "Boss.h"

Boss gameBoss;

//todo consider tuning for difficulty
#define BOSS_TELEPORT_DELAY_ 500
#define BOSS_TELEPORT_TIME_ 100

void initBoss(u8* bossSegment)
{
	u8 position = 0;
	u8 x = 0;
	u8 y = 0;

	while (1)
	{
		//x = 1 -> 17
		//y = 1 -> 9
		x = 1 + cpct_rand() % 17;
		y = 1 + cpct_rand() % 9;

		position = XY_TO_POSITION(x, y);

		if (bossSegment[position] == EMPTY && bossSegment[position + 1] == EMPTY &&
			bossSegment[position + 12] == EMPTY && bossSegment[position + 13] == EMPTY)
		{
			break;
		}
	}

    gameBoss.object.position.x = x * TILE_WIDTH_BYTES;
    gameBoss.object.position.y = y * TILE_HEIGHT;
	gameBoss.object.width = 8;
	gameBoss.object.height = 32;
	gameBoss.health = 100;
    gameBoss.redraw = TRUE;
	gameBoss.timer = 0;
	gameBoss.state = BOSS_STATE_ACTIVE;
	gameBoss.attack = FALSE;
}

void drawBoss()
{
    if(gameBoss.redraw && gameBoss.state)
    {
        u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, gameBoss.object.position.x, gameBoss.object.position.y);
        cpct_drawSprite(G_boss_test, memory, gameBoss.object.width, gameBoss.object.height);

        gameBoss.redraw = FALSE;
    }
}

void updateBossPositionCells(u8 cellType)
{
	u8 x = gameBoss.object.position.x >> 2;
	u8 y = gameBoss.object.position.y >> 4;
	u8 position = XY_TO_POSITION(x, y);

	//top left
	mapCells[position].type = cellType;

	//bottom left
	mapCells[position + 1].type = cellType;

	//top right
	mapCells[position + 12].type = cellType;

	//bottom right
	mapCells[position + 13].type = cellType;
}

void logicBoss()
{
	//don't run the logic if the boss is dead
	if (!gameBoss.state)
	{
		return;
	}

	//check if the boss has any health left
	if (gameBoss.health <= 0)
	{
		//cleanup the boss if 
		gameBoss.redraw = FALSE;

		//cleanup and redraw over boss position
		gameBoss.object.previousPosition.x = gameBoss.object.position.x;
		gameBoss.object.previousPosition.y = gameBoss.object.position.y;
		
		cleanupSegment();

		//set boss to dead
		gameBoss.state = BOSS_STATE_DEAD;

		//500 points for killing the boss
		gameScore += 500;

		return;
	}
	/*dont need this
	if (gameBoss.attack)
	{
		u8 x = gameBoss.object.position.x >> 2;
		u8 y = gameBoss.object.position.y >> 4;
		u8 position = XY_TO_POSITION(x, y);

		//position+1
		//position+13

		gameBoss.attack = FALSE;
	}*/

	//check if timer has elapsed
	if (gameTimer > gameBoss.timer)
	{
		if (gameBoss.state == BOSS_STATE_ACTIVE)
		{
			//teleport boss to new location
			moveBossPosition();

			//update mapCells and redraw cells at old boss position
			updatePreviousBossCells();

			//reset gameBoss timer
			gameBoss.timer = gameTimer + BOSS_TELEPORT_TIME_;

			//displays where the boss will teleport so that it doesnt land on the player
			displayTeleportDestination();

			//change state to teleporting
			gameBoss.state = BOSS_STATE_TELEPORTING;
		}
		else if (gameBoss.state == BOSS_STATE_TELEPORTING)
		{
			//update mapCells with new boss position
			updateBossPositionCells(CELL_TYPE_BOSS);

			//reset gameBoss timer
			gameBoss.timer = gameTimer + BOSS_TELEPORT_DELAY_;

			gameBoss.redraw = TRUE;

			//reset to active state
			gameBoss.state = BOSS_STATE_ACTIVE;
		}
	}
	/* dont need this
	else if (gameTimer + 400 > gameBoss.timer)
	{
		//boss attack
		gameBoss.attack = TRUE;
	}*/
}

void moveBossPosition()
{
	u8 position = 0;
	u8 x = 0;
	u8 y = 0;

	while (1)
	{
		//x = 1 -> 17
		//y = 1 -> 9
		x = 1 + cpct_rand() % 17;
		y = 1 + cpct_rand() % 9;
		
		if (gameBoss.object.position.x == (x * TILE_WIDTH_BYTES) || gameBoss.object.position.y == (y * TILE_HEIGHT))
		{
			//gameboss hasn't moved enough so continue
			continue;
		}

		position = XY_TO_POSITION(x, y);

		if (mapCells[position].type == CELL_TYPE_EMPTY && mapCells[position + 1].type == CELL_TYPE_EMPTY &&
			mapCells[position + 12].type == CELL_TYPE_EMPTY && mapCells[position + 13].type == CELL_TYPE_EMPTY)
		{
			break;
		}
	}

	gameBoss.object.previousPosition.x = gameBoss.object.position.x;
	gameBoss.object.previousPosition.y = gameBoss.object.position.y;

	gameBoss.object.position.x = x * TILE_WIDTH_BYTES;
	gameBoss.object.position.y = y * TILE_HEIGHT;
}

void updatePreviousBossCells()
{
	u8 x = gameBoss.object.previousPosition.x / TILE_WIDTH_BYTES;
	u8 y = gameBoss.object.previousPosition.y / TILE_HEIGHT;
	u8 previousPosition = XY_TO_POSITION(x, y);
	
	//top left
	mapCells[previousPosition].type = CELL_TYPE_EMPTY;
	drawCell(x, y, previousPosition);

	//bottom left
	mapCells[previousPosition + 1].type = CELL_TYPE_EMPTY;
	drawCell(x, y + 1, previousPosition + 1);

	//top right
	mapCells[previousPosition + 12].type = CELL_TYPE_INTERACTABLE;
	mapCells[previousPosition + 12].data.interactable = INTERACTABLE_TYPE_DEATH;
	drawCell(x + 1, y, previousPosition + 12);

	//bottom right
	mapCells[previousPosition + 13].type = CELL_TYPE_INTERACTABLE;
	mapCells[previousPosition + 13].data.interactable = INTERACTABLE_TYPE_DEATH;
	drawCell(x + 1, y + 1, previousPosition + 13);
}

void displayTeleportDestination()
{
	u8 x = gameBoss.object.position.x >> 2;
	u8 y = gameBoss.object.position.y >> 4;
	u8 position = XY_TO_POSITION(x, y);

	updateBossPositionCells(CELL_TYPE_TILE);

	//top left
	mapCells[position].data.tile = TILE_TYPE_BLACK;
	drawCell(x, y, position);

	//bottom left
	mapCells[position + 1].data.tile = TILE_TYPE_BLACK;
	drawCell(x, y+1, position);

	//top right
	mapCells[position + 12].data.tile = TILE_TYPE_BLACK;
	drawCell(x+1, y, position);

	//bottom right
	mapCells[position + 13].data.tile = TILE_TYPE_BLACK;
	drawCell(x+1, y+1, position);
}

void cleanupSegment()
{
	u8 i = 0;

	for (i = 0; i < 240; ++i)
	{
		if ((mapCells[i].type == CELL_TYPE_INTERACTABLE && mapCells[i].data.interactable == INTERACTABLE_TYPE_DEATH)|| mapCells[i].type == CELL_TYPE_BOSS)
		{
			mapCells[i].type = CELL_TYPE_EMPTY;
		}
	}

	//redraw the map
	drawCells();
}