#include "Dialog.h"

#define EXIT_DIALOG_WIDTH_ 38
#define EXIT_DIALOG_HEIGHT_ 20
#define EXIT_DIALOG_X_ 22
#define OBJECTIVE_DIALOG_WIDTH_ 41
#define OBJECTIVE_DIALOG_X_ 24
#define OBJECTIVE_DIALOG_HEIGHT_ 90
#define DIALOG_Y_ 40
#define DIALOG_Y_TEXT_ 45

u8 exitDialog()
{
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, EXIT_DIALOG_X_, DIALOG_Y_);
    cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0) , EXIT_DIALOG_WIDTH_, EXIT_DIALOG_HEIGHT_ );   
    drawString(EXIT_DIALOG_X_ + 1, DIALOG_Y_TEXT_, "exit game  y or n");

    //break out of the loop once the user decides
    while(1)
    {
        if(cpct_isKeyPressed(Key_Y))
        {
			//reset current level to ensure we get game over screen
			currentLevel = 0;

            return TRUE;
        }
        else if(cpct_isKeyPressed(Key_N))
        {            
            //clear just the dialog
            memory = cpct_getScreenPtr(CPCT_VMEM_START, EXIT_DIALOG_X_, DIALOG_Y_);
            cpct_drawSolidBox(memory, cpct_px2byteM0(5, 5) , EXIT_DIALOG_WIDTH_, EXIT_DIALOG_HEIGHT_ );

            return FALSE;
        }
    }
}

void objectiveDialog()
{
	u8 subTypes[2];
    u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, OBJECTIVE_DIALOG_X_, DIALOG_Y_);
    cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0) , OBJECTIVE_DIALOG_WIDTH_, OBJECTIVE_DIALOG_HEIGHT_);
    drawString(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_, "level objectives");

	if (currentLevel == TOP_LEVEL)
	{
		//wirespool item
		subTypes[0] = ITEM_TYPE_OBJECTIVE_WIRE_SPOOL;
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 16, CELL_TYPE_ITEM, subTypes, 1, collectedObjectiveItems, targetItemObjectiveCountTop);

		//fuse item
		subTypes[0] = ITEM_TYPE_OBJECTIVE_FUSE;
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 32, CELL_TYPE_ITEM, subTypes, 1, collectedObjectiveItems2, targetItemObjectiveCountTop);
		
		//transformer and tesla coil
		subTypes[0] = INTERACTABLE_TYPE_TRANSFORMER_0_OFF;
		subTypes[1] = INTERACTABLE_TYPE_TESLA_COIL_0_OFF;
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 48, CELL_TYPE_INTERACTABLE, subTypes, 2, activatedTowerInteractableCount, totalTowerInteractableCount);

		//boss
		if (gameBoss.health)
		{
			displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 64, 4, subTypes, 0, 0, 1);
		}
		else
		{
			displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 64, 4, subTypes, 0, 1, 1);
		}
		
	}
	else if (currentLevel == MIDDLE_LEVEL)
	{
		//gear item
		subTypes[0] = ITEM_TYPE_OBJECTIVE_GEAR;
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 16, CELL_TYPE_ITEM, subTypes, 1, collectedObjectiveItems, targetItemObjectiveCount);
		
		//generator interactable
		subTypes[0] = INTERACTABLE_TYPE_GENERATOR_2_OFF;
		subTypes[1] = INTERACTABLE_TYPE_GENERATOR_3_OFF;
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 32, CELL_TYPE_INTERACTABLE, subTypes, 2, activatedTowerInteractableCount, totalTowerInteractableCount);
		
		//infestation tile
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 48, CELL_TYPE_TILE, subTypes, 0, totalInfestedCellCount-infestedCellCount, targetInfestationCount);
	}
	else if (currentLevel == GROUND_LEVEL)
	{
		//waterpipe item
		subTypes[0] = ITEM_TYPE_OBJECTIVE_WATER_PIPE;
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 16, CELL_TYPE_ITEM, subTypes, 1, collectedObjectiveItems, targetItemObjectiveCount);
		
		//water tank interactable
		subTypes[0] = INTERACTABLE_TYPE_WATER_TANK_0_EMPTY;
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 32, CELL_TYPE_INTERACTABLE, subTypes, 1, activatedTowerInteractableCount, totalTowerInteractableCount);
		
		//valve interactable
		subTypes[0] = INTERACTABLE_TYPE_VALVE;
		displayLine(OBJECTIVE_DIALOG_X_ + 1, DIALOG_Y_TEXT_ + 48, CELL_TYPE_INTERACTABLE, subTypes, 1, activatedValveCount, VALVE_COUNT);
	}

    // wait for any previous key presses to clear first
    while (cpct_isAnyKeyPressed());

    //then wait for another key press
    while (!cpct_isAnyKeyPressed());

	//note: cleanup of the display is expected to be handled by the caller
}

//pipe number of number
void displayLine(u8 x, u8 y, u8 type, u8* subTypes, u8 subTypeCount, u16 currentValue, u16 targetValue)
{
	u8 countAsChar[6];
	u8 i = 0;
	u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);

	if (type == CELL_TYPE_ITEM)
	{
		for (i = 0; i < subTypeCount; ++i)
		{
			cpct_drawSprite(itemGraphicSet[subTypes[i]], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
			memory += 4;
		}
	}
	else if(type == CELL_TYPE_INTERACTABLE)
	{
		for (i = 0; i < subTypeCount; ++i)
		{
			cpct_drawSprite(interactableGraphicSet[subTypes[i]], memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
			memory += 4;
		}
	}
	else if (type == 4)
	{
		cpct_drawSprite(G_objective_dialog_boss, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
	}
	else
	{
		cpct_drawSprite(G_tile_middle_empty_6_infestation, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);
	}

	convertIntToChar(currentValue, countAsChar);
	drawString(x + 11, y + 4, countAsChar);
	drawString(x + 20, y + 4, "of");

	convertIntToChar(targetValue, countAsChar);
	drawString(x + 29, y + 4, countAsChar);


	memory = cpct_getScreenPtr(CPCT_VMEM_START, x + 36, y + 4);
	
	if (currentValue < targetValue)
	{
		cpct_drawSprite(G_objective_dialog_x, memory, 3, 10);		
	}
	else
	{
		cpct_drawSprite(G_objective_dialog_tick, memory, 3, 10);
	}
	
}