#include "Enemy.h"
#include <stdio.h>

void initEnemy(Enemy* enemy, u8 enemyNumber, u8 x, u8 y, u8 type)
{
    //enemy is active
    enemy->destroyed = FALSE;

	//enemy is active so not stunned
	enemy->stunned = FALSE;

	//enemy number is used for identifying where the enemy is within the enemy array
	enemy->enemyNumber = enemyNumber;

    //set it to the type as specified
	enemy->type = type;

    enemy->object.height = ENEMY_HEIGHT;
    enemy->object.width = ENEMY_WIDTH;

    //choose somewhere arbitrary for now, this will be autogenerated based on empty map positions later
    enemy->object.position.x = x * TILE_WIDTH_BYTES;
	enemy->object.position.y = y * TILE_HEIGHT;
	enemy->object.previousPosition.x = x * TILE_WIDTH_BYTES;
	enemy->object.previousPosition.y = y * TILE_HEIGHT;

	//enemy->sourceCellX = x;
	//enemy->sourceCellY = y;
	//enemy->destinationCellX = x;
	//enemy->destinationCellY = y+1;

	enemy->speed = 1;

	enemy->object.velocity.x = 0;
    enemy->object.velocity.y = -enemy->speed;
	enemy->direction = DIRECTION_SOUTH;

	//the cell at the enemy position must be drawn before we can copy it
	drawCell(x, y, XY_TO_POSITION(x, y));

	//set the cell values to where the cell is currently loaded
	enemy->cellX = x;
	enemy->cellXNext = x;
	enemy->cellY = y;
	enemy->cellYNext = y;

	//update map with enemy information
	mapCells[XY_TO_POSITION(x, y)].type = CELL_TYPE_ENEMY;
	mapCells[XY_TO_POSITION(x, y)].data.enemy = enemyNumber;

	//set redraw to true to draw initially
	enemy->redraw = TRUE;

	//copy the content of the background where the enemy sprite will be drawn to save for later
	copyBackground(enemy);
}

void drawEnemy(Enemy* enemy)
{
    u8* memory;

	if (enemy->redraw)
	{
		//dont draw the enemy if it is destroyed
		if (enemy->destroyed)
		{
			return;
		}

		//draw the background only when the position has changed
		if (enemy->object.previousPosition.x != enemy->object.position.x ||
			enemy->object.previousPosition.y != enemy->object.position.y)
		{
			memory = cpct_getScreenPtr(CPCT_VMEM_START, enemy->object.previousPosition.x, enemy->object.previousPosition.y);

			cpct_drawSprite(enemy->backgroundSprite, memory, TILE_WIDTH_BYTES, TILE_HEIGHT);

			copyBackground(enemy);
		}

		memory = cpct_getScreenPtr(CPCT_VMEM_START, enemy->object.position.x, enemy->object.position.y);

		if (enemy->stunned == FALSE)
		{
			cpct_drawSprite(G_enemy_1, memory, enemy->object.width, enemy->object.height);
		}
		else
		{
			//change the enemy colour to white when they get stunned
			cpct_drawSolidBox(memory, cpct_px2byteM0(4, 4), enemy->object.width, enemy->object.height);
		}

		//don't redraw until the enemy has moved
		enemy->redraw = FALSE;
	}
}

void logicEnemy(Enemy* enemy, Actor* playerActor, u8* playerCollisionIndicator, u8* playerCollisionFlickerCount, u8* playerInventory)
{
	//the tiles that the enemy currently occupies
	u8 occupiedXTileLeft = 0;
	u8 occupiedXTileRight = 0;
	u8 occupiedYTileTop = 0;
	u8 occupiedYTileBottom = 0;
	Cell* leftTopCell;
	Cell* rightTopCell;
	Cell* leftBottomCell;
	Cell* rightBottomCell;

	//dont run logic if enemy is destroyed
	if (enemy->destroyed)
	{
		return;
	}

	if (enemy->stunned)
	{
		//check if timer has elapsed
		if (gameTimer > enemy->timer)
		{
			//enemy now not stunned
			enemy->stunned = FALSE;
		}

		//update the previous position before updating the current position with the velocity
		enemy->object.previousPosition.x = enemy->object.position.x;
		enemy->object.previousPosition.y = enemy->object.position.y;
		
		//redraw if enemy stunned
		enemy->redraw = TRUE;

		//enemy cant move while stunned
		return;
	}

	//slow down the logic to improve performance
	if (enemy->speedCount++)
	{
		return;
	}

	if (enemy->type == ENEMY_TYPE_CHASER || enemy->type == ENEMY_TYPE_CHASER_LOW_RANGE)
	{
		eraticEnemy(enemy, &playerActor->object);
	}

	occupiedXTileLeft = (enemy->object.position.x + enemy->object.velocity.x);
	occupiedXTileLeft >>= 2;

	occupiedXTileRight = enemy->object.position.x + enemy->object.width - 1 + enemy->object.velocity.x;
	occupiedXTileRight >>= 2;

	occupiedYTileBottom = (enemy->object.position.y + enemy->object.height - 1 + enemy->object.velocity.y);
	occupiedYTileBottom >>= 4;

	occupiedYTileTop = (enemy->object.position.y + enemy->object.velocity.y);
	occupiedYTileTop >>= 4;

	leftTopCell = &MAPA(occupiedXTileLeft, occupiedYTileTop);
	rightTopCell = &MAPA(occupiedXTileRight, occupiedYTileTop);
	leftBottomCell = &MAPA(occupiedXTileLeft, occupiedYTileBottom);
	rightBottomCell = &MAPA(occupiedXTileRight, occupiedYTileBottom);

	//update the previous position before updating the current position with the velocity
	enemy->object.previousPosition.x = enemy->object.position.x;
	enemy->object.previousPosition.y = enemy->object.position.y;

	/*
	if (enemy->type == ENEMY_TYPE_WALL_HUGGER)
	{
		moveEnemy(enemy);
	}*/

	//test if the enemy is coliding with the player
	if(testObjectIntersection(&playerActor->object, &enemy->object))
	{
		//note there seems to be a potential for a 1 pixel gap displayed when they "colide" but this seems like a compromise im willing to have
#ifdef _WIN32
		printf("enemy collision222!!!!!!!!!!!!!!!!!!!!!\n");
#endif

		//set enemy as stunned from the collision
		enemy->stunned = TRUE;

		//timer is set to 1000 increments of the timer into the future
		enemy->timer = gameTimer + ENEMY_STUN_TIME;

		//while the player is carrying a test tube they do not take damage
		if (*playerInventory & 16)
		{
			return;
		}

		//playerCollisionFlickerCount effectively serves as a player invincibility timer
		if (*playerCollisionFlickerCount == 0)
		{
			*playerCollisionIndicator = TRUE;
			playerActor->timer = gameTimer + 50;

			//number of times to flicker between the player graphic and white
			*playerCollisionFlickerCount = 5;

			//subtract 1 from player health
			if (playerActor->health)
			{
				--playerActor->health;

				//redraw hud to update with new health value
				redrawHud = TRUE;
			}
		}
			
		//enemy now stunned so escape out of logic
		return;
	}	
	
	if (enemy->object.velocity.y > 0)
	{
		//test that the new position is within screen size
		if (enemy->object.position.y + enemy->object.velocity.y <= SCREEN_Y_NO_HUD - enemy->object.height)
		{
			//check to see if the cell is empty
			if(leftBottomCell->type == CELL_TYPE_EMPTY && rightBottomCell->type == CELL_TYPE_EMPTY)
			{
				enemy->object.position.y += enemy->object.velocity.y;
			}
			else if (leftBottomCell->type == CELL_TYPE_ENEMY && rightBottomCell->type == CELL_TYPE_ENEMY)
			{
				//checks if the cell contains an enemy and checks if it is this enemy
				if (leftBottomCell->data.enemy == enemy->enemyNumber && rightBottomCell->data.enemy == enemy->enemyNumber)
				{
					enemy->object.position.y += enemy->object.velocity.y;
				}
			}
			else
			{
				//i would change this but it used less memory then putting it all the above in a single if statement
				goto downBlocked;
			}
		}
		else
		{
			downBlocked:

			switch (enemy->type)
			{
				case ENEMY_TYPE_ROBOT:
					//south to west
					enemy->object.velocity.y = 0;
					enemy->object.velocity.x = -enemy->speed;
					enemy->direction = DIRECTION_WEST;

					//make sure to escape here so we don't check the changed velocity to soon
					return;
				break;

				case ENEMY_TYPE_CHASER:
					//chaser stuck east so we need to change axis
					enemy->object.velocity.y = 0;
					enemy->object.velocity.x = -enemy->speed;
					enemy->direction = DIRECTION_WEST;

					//make sure to escape here so we don't check the changed velocity to soon
					return;
				break;
			}
		}

		//enemy might have moved so redraw
		enemy->redraw = TRUE;
	}
	else if (enemy->object.velocity.y < 0)
	{
		//test that the new position is within screen size
		if (enemy->object.position.y + enemy->object.velocity.y >= 0)
		{
			//check to see if the cell is empty
			if (leftTopCell->type == CELL_TYPE_EMPTY && rightTopCell->type == CELL_TYPE_EMPTY)
			{
				enemy->object.position.y += enemy->object.velocity.y;
			}
			else if (leftTopCell->type == CELL_TYPE_ENEMY && rightTopCell->type == CELL_TYPE_ENEMY)
			{
				//checks if the cell contains an enemy and checks if it is this enemy
				if (leftTopCell->data.enemy == enemy->enemyNumber && rightTopCell->data.enemy == enemy->enemyNumber)
				{
					enemy->object.position.y += enemy->object.velocity.y;
				}
			}
			else
			{
				//i would change this but it used less memory then putting it all the above in a single if statement
				goto upBlocked;
			}
		}
		else
		{
			upBlocked:

			switch (enemy->type)
			{
				case ENEMY_TYPE_ROBOT:
					//north to east
					enemy->object.velocity.y = 0;
					enemy->object.velocity.x = enemy->speed;
					enemy->direction = DIRECTION_EAST;

					//make sure to escape here so we don't check the changed velocity to soon
					return;
				break;

				case ENEMY_TYPE_CHASER:
					//chaser stuck east so we need to change axis
					enemy->object.velocity.y = 0;
					enemy->object.velocity.x = enemy->speed;
					enemy->direction = DIRECTION_EAST;

					//make sure to escape here so we don't check the changed velocity to soon
					return;
				break;
			}
		}

		//enemy might have moved so redraw
		enemy->redraw = TRUE;
	}

	if (enemy->object.velocity.x > 0)
	{
		//test that the new position is within screen size
		if (enemy->object.position.x + enemy->object.velocity.x <= SCREEN_X - enemy->object.width)
		{
			//check to see if the cell is empty
			if (rightTopCell->type == CELL_TYPE_EMPTY && rightBottomCell->type == CELL_TYPE_EMPTY)
			{
				enemy->object.position.x += enemy->object.velocity.x;
			}
			else if (rightTopCell->type == CELL_TYPE_ENEMY && rightBottomCell->type == CELL_TYPE_ENEMY)
			{
				//checks if the cell contains an enemy and checks if it is this enemy
				if (rightTopCell->data.enemy == enemy->enemyNumber && rightBottomCell->data.enemy == enemy->enemyNumber)
				{
					enemy->object.position.x += enemy->object.velocity.x;
				}
			}
			else
			{
				//i would change this but it used less memory then putting it all the above in a single if statement
				goto rightBlocked;
			}
		}
		else
		{
			rightBlocked:

			switch (enemy->type)
			{
				case ENEMY_TYPE_ROBOT:
					//east to south
					enemy->object.velocity.y = enemy->speed;
					enemy->object.velocity.x = 0;
					enemy->direction = DIRECTION_SOUTH;

					//make sure to escape here so we don't check the changed velocity to soon
					return;
				break;

				case ENEMY_TYPE_CHASER:
					//chaser stuck east so we need to change axis
					enemy->object.velocity.y = enemy->speed;
					enemy->object.velocity.x = 0;
					enemy->direction = DIRECTION_SOUTH;

					//make sure to escape here so we don't check the changed velocity to soon
					return;
				break;
			}
		}

		//enemy might have moved so redraw
		enemy->redraw = TRUE;
	}
	else if (enemy->object.velocity.x < 0)
	{
		//test that the new position is within screen size
		if (enemy->object.position.x + enemy->object.velocity.x >= 0)
		{
			//check to see if the cell is empty
			if(leftTopCell->type == CELL_TYPE_EMPTY && leftBottomCell->type == CELL_TYPE_EMPTY)
			{
				enemy->object.position.x += enemy->object.velocity.x;
			}
			else if (leftTopCell->type == CELL_TYPE_ENEMY && leftBottomCell->type == CELL_TYPE_ENEMY)
			{
				//checks if the cell contains an enemy and checks if it is this enemy
				if (leftTopCell->data.enemy == enemy->enemyNumber && leftBottomCell->data.enemy == enemy->enemyNumber)
				{
					enemy->object.position.x += enemy->object.velocity.x;
				}
			}
			else
			{
				//i would change this but it used less memory then putting it all the above in a single if statement
				goto leftBlocked;
			}
		}
		else
		{
			leftBlocked:

			switch (enemy->type)
			{
				case ENEMY_TYPE_ROBOT:
					//west to north
					enemy->object.velocity.y = -enemy->speed;
					enemy->object.velocity.x = 0;
					enemy->direction = DIRECTION_NORTH;

					//make sure to escape here so we don't check the changed velocity to soon
					return;
				break;

				case ENEMY_TYPE_CHASER:
					//chaser stuck east so we need to change axis
					enemy->object.velocity.y = -enemy->speed;
					enemy->object.velocity.x = 0;
					enemy->direction = DIRECTION_NORTH;

					//make sure to escape here so we don't check the changed velocity to soon
					return;
				break;
			}
		}

		//enemy might have moved so redraw
		enemy->redraw = TRUE;
	}

	//updates where the enemy currently resides in the cellGrid
	
	//todo: why not try rewrite just this in z80 asm
	updateEnemyCellPosition(enemy);
}

//currently left here for reference but generally unwanted and probably no longer functional
/*
void moveEnemy(Enemy* enemy)
{
	//update the source cellX position based on the current enemy position
	enemy->sourceCellX = enemy->object.position.x >> 2;
	enemy->sourceCellY = enemy->object.position.y >> 4;
	
	//if there is any leftover for y then go back up one
	if (enemy->object.position.y & 15)
	{
		--enemy->sourceCellY;
	}

	if (enemy->object.position.x & 3)
	{
		--enemy->sourceCellX;
	}

	switch (enemy->direction)
	{
		case DIRECTION_NORTH:
		{
			//reached the destination cell
			if (enemy->sourceCellY == enemy->destinationCellY)
			{
				//we either continue onwards or we change direction
				if (enemy->destinationCellY - 1 >= 0 && MAPA(enemy->sourceCellX, (enemy->destinationCellY - 1)).type == CELL_TYPE_EMPTY)
				{
					//above is clear so continue
					enemy->destinationCellY = enemy->sourceCellY - 1;
				}
				else if (enemy->sourceCellX + 1 < SCREEN_TILES_X && MAPA((enemy->sourceCellX + 1), enemy->destinationCellY).type == CELL_TYPE_EMPTY)
				{
					//east is clear so change direction
					enemy->direction = DIRECTION_EAST;
				}
				else if (enemy->sourceCellX - 1 >= 0 && MAPA((enemy->sourceCellX - 1), enemy->destinationCellY).type == CELL_TYPE_EMPTY)
				{
					//west is clear so change direction
					enemy->direction = DIRECTION_WEST;
				}
				else
				{
					//all blocked so go back south
					enemy->direction = DIRECTION_SOUTH;

					//change destination to 1 cell south
					enemy->destinationCellY = enemy->sourceCellY + 1;
				}
			}
			else
			{
				enemy->object.velocity.y = -enemy->speed;
				enemy->object.velocity.x = 0;
			}
		}
		break;

		case DIRECTION_SOUTH:
		{
			//reached the destination cell
			if (enemy->sourceCellY == enemy->destinationCellY)
			{
				//we either continue onwards or we change direction
				if (enemy->destinationCellY + 1 < SCREEN_TILES_Y && MAPA(enemy->sourceCellX, (enemy->destinationCellY + 1)).type == CELL_TYPE_EMPTY)
				{
					//below is clear so continue
					enemy->destinationCellY = enemy->sourceCellY + 1;
				}
				else if (enemy->sourceCellX + 1 < SCREEN_TILES_X && MAPA((enemy->sourceCellX + 1), enemy->destinationCellY).type == CELL_TYPE_EMPTY)
				{
					//east is clear so change direction
					enemy->direction = DIRECTION_EAST;
				}
				else if (enemy->sourceCellX - 1 >= 0 && MAPA((enemy->sourceCellX - 1), enemy->destinationCellY).type == CELL_TYPE_EMPTY)
				{
					//west is clear so change direction
					enemy->direction = DIRECTION_WEST;
				}
				else
				{
					//all blocked so go back north
					enemy->direction = DIRECTION_NORTH;

					//change destination to 1 cell north
					enemy->destinationCellY = enemy->sourceCellY - 1;
				}
			}
			else
			{
				enemy->object.velocity.y = enemy->speed;
				enemy->object.velocity.x = 0;
			}
		}
		break;

		case DIRECTION_EAST:
		{
			//reached the destination cell
			if (enemy->sourceCellX == enemy->destinationCellX)
			{
				//we either continue onwards or we change direction
				if (enemy->destinationCellX + 1 < SCREEN_TILES_X && MAPA((enemy->destinationCellX + 1), enemy->sourceCellY).type == CELL_TYPE_EMPTY)
				{
					//east is clear so continue
					enemy->destinationCellX = enemy->sourceCellX + 1;
				}
				else if (enemy->sourceCellY + 1 < SCREEN_TILES_Y && MAPA(enemy->destinationCellX, (enemy->sourceCellY + 1)).type == CELL_TYPE_EMPTY)
				{
					//south is clear so change direction
					enemy->direction = DIRECTION_SOUTH;
				}
				else if (enemy->sourceCellY - 1 >= 0 && MAPA(enemy->destinationCellX, (enemy->sourceCellY - 1)).type == CELL_TYPE_EMPTY)
				{
					//north is clear so change direction
					enemy->direction = DIRECTION_NORTH;
				}
				else
				{
					//all blocked so go back west
					enemy->direction = DIRECTION_WEST;
				}
			}
			else
			{
				enemy->object.velocity.y = 0;
				enemy->object.velocity.x = enemy->speed;
			}
		}
		break;

		case DIRECTION_WEST:
		{
			//reached the destination cell
			if (enemy->sourceCellX == enemy->destinationCellX)
			{
				//we either continue onwards or we change direction
				if (enemy->destinationCellX - 1 >= 0 && MAPA((enemy->destinationCellX - 1), enemy->sourceCellY).type == CELL_TYPE_EMPTY)
				{
					//west is clear so continue
					enemy->destinationCellX = enemy->sourceCellX - 1;
				}
				else if (enemy->sourceCellY - 1 >= 0 && MAPA(enemy->destinationCellX, (enemy->sourceCellY - 1)).type == CELL_TYPE_EMPTY)
				{
					//north is clear so change direction
					enemy->direction = DIRECTION_NORTH;
				}
				else if (enemy->sourceCellY + 1 < SCREEN_TILES_Y && MAPA(enemy->destinationCellX, (enemy->sourceCellY + 1)).type == CELL_TYPE_EMPTY)
				{
					//south is clear so change direction
					enemy->direction = DIRECTION_SOUTH;
				}
				else
				{
					//all blocked so go back east
					enemy->direction = DIRECTION_EAST;
				}
			}
			else
			{
				enemy->object.velocity.y = 0;
				enemy->object.velocity.x = -enemy->speed;
			}
		}
		break;
	}
}
*/

void eraticEnemy(Enemy* enemy, Object* playerObject)
{
	u8 playerXPosition = playerObject->position.x >> 2;
	u8 playerYPosition = playerObject->position.y >> 4;
	u8 enemyXPosition = enemy->object.position.x >> 2;
	u8 enemyYPosition = enemy->object.position.y >> 4;

	//this ensures that the enemy is only in a single cell before potentially changing direction
	if (enemy->cellX != enemy->cellXNext || enemy->cellY != enemy->cellYNext)
	{
		return;
	}

	//a low range enemy can be run away from effectively it has short vision
	if (enemy->type == ENEMY_TYPE_CHASER_LOW_RANGE)
	{
		if (playerXPosition - enemyXPosition > 7 || playerXPosition - enemyXPosition < -7)
		{
			enemy->object.velocity.x = 0;
			enemy->object.velocity.y = enemy->speed;
			enemy->direction = DIRECTION_NORTH;
			return;
		}

		if (playerYPosition - enemyYPosition > 5 || playerYPosition - enemyYPosition < -5)
		{
			enemy->object.velocity.x = enemy->speed;
			enemy->object.velocity.y = 0;
			enemy->direction = DIRECTION_EAST;
			return;
		}
	}
	
	if (enemy->direction == DIRECTION_EAST || enemy->direction == DIRECTION_WEST)
	{
		if (enemyXPosition < playerXPosition)
		{
			enemy->object.velocity.x = enemy->speed;
			enemy->direction = DIRECTION_EAST;
		}
		else if (enemyXPosition > playerXPosition)
		{
			enemy->object.velocity.x = -enemy->speed;
			enemy->direction = DIRECTION_WEST;
		}
		else
		{
			enemy->object.velocity.x = 0;
			enemy->object.velocity.y = enemy->speed;
			enemy->direction = DIRECTION_NORTH;
		}
	}


	if (enemy->direction == DIRECTION_NORTH || enemy->direction == DIRECTION_SOUTH)
	{
		//north or south
		if (enemyYPosition < playerYPosition)
		{
			enemy->object.velocity.y = enemy->speed;
			enemy->direction = DIRECTION_SOUTH;
		}
		else if (enemyYPosition > playerYPosition)
		{
			enemy->object.velocity.y = -enemy->speed;
			enemy->direction = DIRECTION_NORTH;
		}
		else
		{
			enemy->object.velocity.x = enemy->speed;
			enemy->object.velocity.y = 0;
			enemy->direction = DIRECTION_EAST;
		}
	}
}

void updateEnemyCellPosition(Enemy* enemy)
{
	u8 cellXMin = enemy->object.position.x >> 2;
	u8 cellXMax = (enemy->object.position.x + enemy->object.width - 1);

	u8 cellYMin = enemy->object.position.y >> 4;
	u8 cellYMax = (enemy->object.position.y + enemy->object.height - 1);
	u8 position = XY_TO_POSITION(enemy->cellX, enemy->cellY);

	//need to assign enemy->enemyNumber to a variable because using it to set the cell data union 
	//results in unexpected values when running on the CPC (possible SDCC compiler bug)
	u8 enemyNumber = enemy->enemyNumber;

	//divide x by 4
	cellXMax >>= 2;

	//divide y by 16
	cellYMax >>= 4;

	
	//maybe this would be good
	//if (enemy->object.previousPosition.y == enemy->object.position.y &&
	//	enemy->object.previousPosition.x == enemy->object.position.x)
	//{
	//	//no change dont do anything
	//	return;
	//}
	
	//reset current cells enemy currently occupies to empty
	mapCells[position].type = CELL_TYPE_EMPTY;

	//reset tile to empty for redraw
	mapCells[position].data.tile = emptyTile;

	position = XY_TO_POSITION(enemy->cellXNext, enemy->cellYNext);
	mapCells[position].type = CELL_TYPE_EMPTY;

	//reset tile to empty for redraw
	mapCells[position].data.tile = emptyTile;

	if (enemy->object.previousPosition.x < enemy->object.position.x)
	{
		//has gone left
		if (cellXMin != cellXMax)
		{									
			enemy->cellXNext = cellXMin;
			enemy->cellX = cellXMax;
		}
		else
		{
			//min max equal so set both to either
			enemy->cellX = cellXMin;
			enemy->cellXNext = cellXMin;
		}
	}
	else if (enemy->object.previousPosition.x > enemy->object.position.x)
	{
		//has gone right
		if (cellXMin != cellXMax)
		{
			enemy->cellXNext = cellXMax;
			enemy->cellX = cellXMin;
		}
		else
		{
			//min max equal so set both to either
			enemy->cellX = cellXMin;
			enemy->cellXNext = cellXMin;
		}
	}
	else if (enemy->object.previousPosition.y < enemy->object.position.y)
	{
		//has gone up
		if (cellYMin != cellYMax)
		{
			enemy->cellYNext = cellYMin;
			enemy->cellY = cellYMax;
		}
		else
		{
			//min max equal so set both to either
			enemy->cellY = cellYMin;
			enemy->cellYNext = cellYMin;
		}
	}
	else if (enemy->object.previousPosition.y > enemy->object.position.y)
	{
		//has gone down
		if (cellYMin != cellYMax)
		{
			enemy->cellYNext = cellYMax;
			enemy->cellY = cellYMin;
		}
		else
		{
			//min max equal so set both to either
			enemy->cellY = cellYMin;
			enemy->cellYNext = cellYMin;
		}
	}

#ifdef _WIN32
	printf("(%i) src(%i,%i) -> dest(%i,%i)\n", enemy->enemyNumber, enemy->cellX,enemy->cellY, enemy->cellXNext,enemy->cellYNext);
#endif

	//update cells the enemy occupies with enemy indicator
	position = XY_TO_POSITION(enemy->cellX, enemy->cellY);
	mapCells[position].type = CELL_TYPE_ENEMY;

	//using enemy->enemyNumber results in bad values in the union
	mapCells[position].data.enemy = enemyNumber;

	position = XY_TO_POSITION(enemy->cellXNext, enemy->cellYNext);
	mapCells[position].type = CELL_TYPE_ENEMY;

	//using enemy->enemyNumber results in bad values in the union
	mapCells[position].data.enemy = enemyNumber;
}

void killEnemy(Enemy* enemy)
{
	u8 position = 0;

	//set enemy to destroyed so it isn't displayed and logic isn't run
	enemy->destroyed = TRUE;

	//clear enemy from mapCells
	position = XY_TO_POSITION(enemy->cellX, enemy->cellY);
	mapCells[position].type = CELL_TYPE_EMPTY;

	//reset tile to empty for redraw
	mapCells[position].data.tile = emptyTile;
	
	//redraw cell where enemy was
	drawCell(enemy->cellX, enemy->cellY, position);

	position = XY_TO_POSITION(enemy->cellXNext, enemy->cellYNext);
	mapCells[position].type = CELL_TYPE_EMPTY;

	//reset tile to empty for redraw
	mapCells[position].data.tile = emptyTile;

	//redraw cell where enemy was
	drawCell(enemy->cellXNext, enemy->cellYNext, position);
	
}