#include "Hud.h"

#define HUD_X_POSITION_ 0
#define HUD_X_POSITION_TWO_ 64
#define HUD_Y_POSITION_ 192
#define HUD_WIDTH_ONE_ 64
#define HUD_WIDTH_TWO_ 16
#define HUD_HEIGHT_ 8

#define HUD_SCORE_START_ 0
#define LIFECOUNT_START_ 19
#define HEALTH_START_ 25
#define OBJECTIVE_ITEM_START_ 40
#define OBJECTIVE_ITEM2_START_ 47
#define CURRENT_WEAPON_AMMO_START_ 54
#define INVENTORY_KEY_RED_START_ 66
#define INVENTORY_KEY_GREEN_START_ 68
#define INVENTORY_KEY_BLUE_START_ 70
#define INVENTORY_SPANNER_START_ 72
#define INVENTORY_TEST_TUBE_START_ 74
#define HUD_ITEM_WIDTH_BYTES_ 4

u8 redrawHud = TRUE;
u8 backgroundColour_;

void initHud()
{
    redrawHud = TRUE;
    backgroundColour_ = cpct_px2byteM0(12, 12);
}

void drawHud()
{
    if(redrawHud)
    {
        u8* hudVideoMemory;
        u8 livesAscii[2];
        u8 healthText[6];
        u8 scoreText[6];
        u8 objectiveText[6];
        u8 ammoText[6];

		//draw the hud strip on the background in two parts
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_, HUD_Y_POSITION_);
		cpct_drawSolidBox(hudVideoMemory, backgroundColour_, HUD_WIDTH_ONE_, HUD_HEIGHT_);

		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HUD_X_POSITION_TWO_, HUD_Y_POSITION_);
		cpct_drawSolidBox(hudVideoMemory, backgroundColour_, HUD_WIDTH_TWO_, HUD_HEIGHT_);
		
        //highscore
		convertIntToChar(gameScore, scoreText);
        drawString(HUD_SCORE_START_, HUD_Y_POSITION_, scoreText);

        //life count
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, LIFECOUNT_START_, HUD_Y_POSITION_);
		cpct_drawSprite(G_hud_lives, hudVideoMemory, 3, 8);

        livesAscii[0] = player.lives + ASCII_TO_NUMBER_OFFSET;
		livesAscii[1] = '\0';
        drawString(LIFECOUNT_START_ + 3, HUD_Y_POSITION_, livesAscii);

        //player health
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, HEALTH_START_, HUD_Y_POSITION_);
		cpct_drawSprite(G_hud_health, hudVideoMemory, 3, 8);
		convertIntToChar(player.health, healthText);
        drawString(HEALTH_START_ + 3, HUD_Y_POSITION_, healthText);		

		//objective items
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, OBJECTIVE_ITEM_START_, HUD_Y_POSITION_);

		if (currentLevel == GROUND_LEVEL)
		{			
			cpct_drawSprite(G_hud_water_pipe, hudVideoMemory, 2, 8);
		}
		else if (currentLevel == MIDDLE_LEVEL)
		{
			cpct_drawSprite(G_hud_gear, hudVideoMemory, 2, 8);
		}
		else
		{
			//top level
			cpct_drawSprite(G_hud_wire_spool, hudVideoMemory, 2, 8);

			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, OBJECTIVE_ITEM2_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_fuse, hudVideoMemory, 2, 8);
		}

		//objective item text
        convertIntToChar(collectedObjectiveItems, objectiveText);
        drawString(OBJECTIVE_ITEM_START_ + 2, HUD_Y_POSITION_, objectiveText);

		//objective item 2 text
		if (currentLevel == TOP_LEVEL)
		{
			convertIntToChar(collectedObjectiveItems2, objectiveText);
			drawString(OBJECTIVE_ITEM2_START_ + 2, HUD_Y_POSITION_, objectiveText);
		}

        //current (projectile) weapon ammo
		hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, CURRENT_WEAPON_AMMO_START_, HUD_Y_POSITION_);

        if(player.currentWeapon == PROJECTILE_WEAPON)
        {
			cpct_drawSprite(G_hud_projectile_ammo, hudVideoMemory, 2, 8);
			convertIntToChar(player.projectileAmmo, ammoText);
			drawString(CURRENT_WEAPON_AMMO_START_ + 2, HUD_Y_POSITION_, ammoText);
        }
        else if(player.currentWeapon == FLAMETHROWER_WEAPON)
        {
			cpct_drawSprite(G_hud_flamethrower_ammo, hudVideoMemory, 2, 8);
			convertIntToChar(player.flamethrowerAmmo, ammoText);
			drawString(CURRENT_WEAPON_AMMO_START_ + 2, HUD_Y_POSITION_, ammoText);
        }
        else if(player.currentWeapon == LASER_WEAPON)
        {
			cpct_drawSprite(G_hud_laser_ammo, hudVideoMemory, 2, 8);
			convertIntToChar(player.laserAmmo, ammoText);
			drawString(CURRENT_WEAPON_AMMO_START_ + 2, HUD_Y_POSITION_, ammoText);
        }

		//display a red key if the player is carrying one
		if (player.inventory & INVENTORY_ITEM_KEY)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, INVENTORY_KEY_RED_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_key, hudVideoMemory, 2, 8);
		}

        //display a green key if the player is carrying one
		if (player.inventory & INVENTORY_ITEM_KEY_GREEN)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, INVENTORY_KEY_GREEN_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_key_green, hudVideoMemory, 2, 8);
		}

        //display a blue key if the player is carrying one
		if (player.inventory & INVENTORY_ITEM_KEY_BLUE)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, INVENTORY_KEY_BLUE_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_key_blue, hudVideoMemory, 2, 8);
		}

        //display a spanner if the player is carrying one
		if (player.inventory & INVENTORY_ITEM_SPANNER)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, INVENTORY_SPANNER_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_spanner, hudVideoMemory, 2, 8);
		}

		//display a test tube if the player is carrying one
		if (player.inventory & INVENTORY_ITEM_TEST_TUBE)
		{
			hudVideoMemory = cpct_getScreenPtr(CPCT_VMEM_START, INVENTORY_TEST_TUBE_START_, HUD_Y_POSITION_);
			cpct_drawSprite(G_hud_test_tube, hudVideoMemory, 2, 8);
		}		

		redrawHud = FALSE;
    }
}