#include "Map.h"
#include <stdio.h>

#ifdef _WIN32
#include <stdlib.h>
#endif

#define BIG_MAP_SIZE_ 3840
#define BLOCK_SIZE_ 960
#define DEATH_TRAP_MAX_COUNT_ 6

#define HORIZONTAL_INTERACTABLE_ 0
#define VERTICIAL_INTERACTABLE_ 1
#define SEGMENT_TRANSITION_RANDOM_ 0
#define SEGMENT_TRANSITION_NEXT_ 1

//map arrays
u8* mapArray = NULL;
u8 mapRedraw = TRUE;

u8* segments[SEGMENT_COUNT];

u8 bigMap[BIG_MAP_SIZE_];

DeathTrap deathTraps[DEATH_TRAP_MAX_COUNT_];

void initMap(u8 seed)
{
	u8 exitBlock1 = 0;
	u8 exitBlock2 = 0;
	u8 levelExitBlock = 0;
	u8 wallType = 0;

	mapRedraw = TRUE;

	if (currentLevel == MIDDLE_LEVEL)
	{
		wallType = TILE_TYPE_WALL_MIDDLE;
	}
	else if (currentLevel == TOP_LEVEL)
	{
		wallType = TILE_TYPE_WALL_TOP;
	}

	//----------------------------
	//--  MAZE BLOCK TOP LEFT|0 --
	//----------------------------
	//generate the maze for the game
	generateMaze(SCREEN_TILES_X-1, 1, wallType, seed + 10);

    //initialise the big map array to 0
    cpct_memset(bigMap, 0, BIG_MAP_SIZE_);
	
    //copy the small array into the big array   
	bigMapTallCombined(mazeGrid, bigMap, SCREEN_TILES_Y);		

	//----------------------------
	//-- MAZE BLOCK TOP RIGHT|1 --
	//----------------------------
	//generate the second block maze
	generateMaze(0, 1, wallType, seed + 20);

	//add an exit to the maze block 1 (south for now)
	exitBlock1 = addExit(DIRECTION_SOUTH, wallType);

	//copy the small array into the big array after the previous block
	bigMapTallCombined(mazeGrid, bigMap + BLOCK_SIZE_, SCREEN_TILES_Y);


	//----------------------------
	//-- MAZE BLOCK BOT RIGHT|2 --
	//----------------------------
	//generate the third block maze
	generateMaze(exitBlock1, 0, wallType, seed + 30);

	//add an exit to the maze block 2 (east for now)
	exitBlock2 = addExit(DIRECTION_EAST, wallType);

	//copy the small array into the big array after the previous block
	bigMapTallCombined(mazeGrid, bigMap + BLOCK_SIZE_ + BLOCK_SIZE_, SCREEN_TILES_Y);


	//----------------------------
	//--  MAZE BLOCK BOT LEFT|3 --
	//----------------------------
	//generate the forth block maze
	generateMaze(SCREEN_TILES_X - 1, exitBlock2, wallType, seed + 40);

	//add an exit to the maze block 3 (west for now)
	addExit(DIRECTION_WEST, wallType);

	//add a final level exit at the east (bottom left segment)
	levelExitBlock = addExit(DIRECTION_EAST, wallType);

	//copy the small array into the big array after the previous block
	bigMapTallCombined(mazeGrid, bigMap + BLOCK_SIZE_ + BLOCK_SIZE_ + BLOCK_SIZE_, SCREEN_TILES_Y);

    //mazeGrid is a block that is divided into segments
	//block 0
    segments[0] = bigMap;
    segments[1] = bigMap + 240;
    segments[4] = bigMap + 480;
    segments[5] = bigMap + 720;

	//adding a door to the first maze block
	//the generated exit is for the original map rather than the expanded map so we multiply by 2
	addDoor(DIRECTION_WEST, segments[1], SCREEN_TILES_X - 1, 1 * 2, INTERACTABLE_TYPE_DOOR_RED);

	//add the segments for the second block
	//block 1
	segments[2] = bigMap + BLOCK_SIZE_;
	segments[3] = bigMap + BLOCK_SIZE_ + 240;
	segments[6] = bigMap + BLOCK_SIZE_ + 480;
	segments[7] = bigMap + BLOCK_SIZE_ + 720;

	//adding a door to the second maze block
	
	if (exitBlock1 * 2 >= SCREEN_TILES_X)
	{
		//when the block is >= 20 we have to subtract from the size of the segment (20) as we are in the right segment
		addDoor(DIRECTION_SOUTH, segments[7], (exitBlock1 * 2) - SCREEN_TILES_X, SCREEN_TILES_Y - 1, INTERACTABLE_TYPE_DOOR_GREEN);
	}
	else
	{
		//this is in the left segment
		//the generated exit is for the original map rather than the expanded map so we multiply by 2
		addDoor(DIRECTION_SOUTH, segments[6], exitBlock1 * 2, SCREEN_TILES_Y - 1, INTERACTABLE_TYPE_DOOR_GREEN);
	}

	//add the segments for the fourth block
	//block 3
	segments[8] = bigMap + BLOCK_SIZE_ + BLOCK_SIZE_ + BLOCK_SIZE_;
	segments[9] = bigMap + BLOCK_SIZE_ + BLOCK_SIZE_ + BLOCK_SIZE_ + 240;
	segments[12] = bigMap + BLOCK_SIZE_ + BLOCK_SIZE_ + BLOCK_SIZE_ + 480;
	segments[13] = bigMap + BLOCK_SIZE_ + BLOCK_SIZE_ + BLOCK_SIZE_ + 720;

	//final level exit
	if (levelExitBlock * 2 >= SCREEN_TILES_Y)
	{
		//when block is >= 12 we have to subtract from the size of the segment (12) as we are in the bottom segment
		addDoor(DIRECTION_EAST, segments[12], 0, levelExitBlock * 2 - SCREEN_TILES_Y, INTERACTABLE_TYPE_DOOR_OBJECTIVE);
	}
	else
	{
		//the generated exit is for the original map rather than the expanded map so we multiply by 2
		addDoor(DIRECTION_EAST, segments[14], 0, levelExitBlock * 2, INTERACTABLE_TYPE_DOOR_OBJECTIVE);
	}

	//add the segments for the third block
	//block 2
	segments[10] = bigMap + BLOCK_SIZE_ + BLOCK_SIZE_;
	segments[11] = bigMap + BLOCK_SIZE_ + BLOCK_SIZE_ + 240;
	segments[14] = bigMap + BLOCK_SIZE_ + BLOCK_SIZE_ + 480;
	segments[15] = bigMap + BLOCK_SIZE_ + BLOCK_SIZE_ + 720;

    //adding door into block 2 (effectively the last block)
	if (exitBlock2 * 2 >= SCREEN_TILES_Y)
	{
		//when block is >= 12 we have to subtract from the size of the segment (12) as we are in the bottom segment
		addDoor(DIRECTION_EAST, segments[14], 0, exitBlock2 * 2 - SCREEN_TILES_Y, INTERACTABLE_TYPE_DOOR_BLUE);
	}
	else
	{
		//the generated exit is for the original map rather than the expanded map so we multiply by 2
		addDoor(DIRECTION_EAST, segments[14], 0, exitBlock2 * 2, INTERACTABLE_TYPE_DOOR_BLUE);
	}
		
	//start at segment 0
	currentSegment = 0;
	    
	//populate the specified level with items and interactables and environment
	populateLevel(currentLevel);

	//write out to file for testing
#ifdef _WIN32
	FILE* file = fopen("bigMapOut.txt", "w");
	
	fwrite(bigMap, sizeof(u8), BIG_MAP_SIZE_, file);

	fclose(file);

#endif

	//start at block 0
    initCells(segments[0]);

	//setup death traps for the cells
	//setupDeathTraps();
	
	//setup enemies for the current segment
	distributeEnemies();
}

void drawMap()
{
    if(mapRedraw)
    {
        //draw cells can cover all this 
        drawCells();
        
        mapRedraw = FALSE;
    }    
}

void bigMapTallCombined(u8* oneArray, u8* twoArray, u8 rowLength)
{
	u16 row1 = 0;
	u16 row2 = rowLength;
	u16 row3 = 480;
	u16 row4 = 480 + rowLength;
	u8 i = 0;
	u8 j = 0;


	for (i = 0; i<240; ++i)
	{
		u8 cellValue;

		//once we reach the end of the row of the threeArray we need to jump down to the next row of two
		if (i != 0 && i % (rowLength) == 0)
		{
			row1 += rowLength;
			row2 += rowLength;
			row3 += rowLength;
			row4 += rowLength;
		}


		//iterate through each of the cells of oneArray
		cellValue = oneArray[i];

		if (i % (rowLength) < rowLength / 2)
		{
			//fill each block in the two array with a value from the one array
			for (j = 0; j<2; ++j)
			{
				twoArray[row1++] = cellValue;
				twoArray[row2++] = cellValue;
			}
		}
		else
		{
			//fill each block in the two array with a value from the one array
			for (j = 0; j<2; ++j)
			{
				twoArray[row3++] = cellValue;
				twoArray[row4++] = cellValue;
			}
		}
	}
}

void moveSegment(u8 direction, Object* playerObject)
{
	u8 xTileLeft = playerObject->position.x >> 2;
	u8 xTileRight = (playerObject->position.x + playerObject->width - 1);
	u8 yTileTop = (playerObject->position.y) >> 4;
	u8 yTileBottom = (playerObject->position.y + playerObject->height - 1);
	u8 previousSegment = currentSegment;

	xTileRight >>= 2;
	yTileBottom >>= 4;

	switch (direction)
	{
		case DIRECTION_SOUTH:
		{
			if (currentSegment + SEGMENTS_X < SEGMENT_COUNT)
			{
				u8* segmentMapArray = segments[currentSegment + SEGMENTS_X];

				//ytile is always 1
				//x tile is from the playerObject
				
				if (segmentMapArray[xTileLeft * SCREEN_TILES_Y + 0] >= 100 && segmentMapArray[xTileRight * SCREEN_TILES_Y + 0] >= 100)
				{
					//segment is clear from this position
					currentSegment += SEGMENTS_X;

					//reposition player on map (only y needs to change)
					playerObject->position.y = 0;
					playerObject->previousPosition.y = -1;
				}
				else
				{
					//path blocked
					return;
				}						
			}
			else
			{
				//no change
				return;
			}
		}
		break;

		case DIRECTION_NORTH:
		{
			if (currentSegment - SEGMENTS_X >= 0)
			{
				u8* segmentMapArray = segments[currentSegment - SEGMENTS_X];

				//ytile is always SCREEN_TILES_Y-1
				//x tile is from the playerObject

				if (segmentMapArray[xTileLeft * SCREEN_TILES_Y + SCREEN_TILES_Y - 1] >= 100 && segmentMapArray[xTileRight * SCREEN_TILES_Y + SCREEN_TILES_Y - 1] >= 100)
				{
					//segment is clear from this position
					currentSegment -= SEGMENTS_X;

					//reposition player on map (only y needs to change)
					playerObject->position.y = SCREEN_Y_NO_HUD - playerObject->height;
					playerObject->previousPosition.y = -1;
				}
				else
				{
					//path blocked
					return;
				}
			}
			else
			{
				//no change
				return;
			}
		}
		break;

		case DIRECTION_EAST:
		{
			if (currentSegment + 1 < SEGMENT_COUNT)
			{
                u8* segmentMapArray = segments[currentSegment + 1];

                //xTile is 0
                //yTile is from playerObject

                if (segmentMapArray[0 * SCREEN_TILES_Y + yTileTop] >= 100 && segmentMapArray[0 * SCREEN_TILES_Y + yTileBottom] >= 100)
                {
                    //segment is clear from this position
					currentSegment += 1;

					//reposition player on map (only x needs to change)
					playerObject->position.x = 0;
					playerObject->previousPosition.x = -1;
				}
				else
				{
					//path blocked
					return;
				}
			}
			else
			{
				//no change
				return;
			}
		}
		break;

		case DIRECTION_WEST:
		{
            if (currentSegment - 1 >= 0)
			{
                u8* segmentMapArray = segments[currentSegment - 1];
                
                //xTile is SCREEN_TILES_X-1
                //yTile is from playerObject

                if (segmentMapArray[(SCREEN_TILES_X-1) * SCREEN_TILES_Y + yTileTop] >= 100 && segmentMapArray[(SCREEN_TILES_X-1) * SCREEN_TILES_Y + yTileBottom] >= 100)
                {
                    //segment is clear from this position
					currentSegment -= 1;

					//reposition player on map (only x needs to change)
					playerObject->position.x = SCREEN_X - playerObject->width;
					playerObject->previousPosition.x = -1;
				}
				else
				{
					//path blocked
					return;
				}
			}
			else
			{
				//no change
				return;
			}
		}
		break;
	}

	//copy changes in the cell grid back to the segment (ie the big map) so we cant pickup/use/activate things twice
	copyCellChangesToSegment(previousSegment);

	//reinitalise the cells of the grid
	initCells(segments[currentSegment]);

	//update death traps
	//setupDeathTraps();

	//no enemies required for boss segment
	if (!(currentLevel == TOP_LEVEL && currentSegment == BOSS_SEGMENT))
	{
		//redistribute the enemies into the new segment
		distributeEnemies();
	}
	else
	{
		//ensure boss is redrawn on moving in and out of the segment
		gameBoss.redraw = TRUE;

		//add the boss types to mapCells
		updateBossPositionCells(CELL_TYPE_BOSS);
	}

	//the player will only be across on tile vertically but could be across two tiles horizontally
	//recalculate tile values and process the cells
	xTileLeft = playerObject->position.x >> 2;
	xTileRight = (playerObject->position.x + playerObject->width - 1);
	yTileTop = (playerObject->position.y) >> 4;

	processCell(xTileLeft, yTileTop);
	processCell(xTileRight, yTileTop);
	
	//save player position for if they lose a life
	playerEntryX = playerObject->position.x;
	playerEntryY = playerObject->position.y;
	
	//set map for redraw
	mapRedraw = TRUE;
}

void copyCellChangesToSegment(u8 segment)
{
	u8 i = 0;
	u8* segmentMapArray = segments[segment];

	for (i = 0; i < 240; ++i)
	{
		if (mapCells[i].type == CELL_TYPE_EMPTY)
		{
			segmentMapArray[i] = EMPTY;
		}
		else if (mapCells[i].type == CELL_TYPE_INTERACTABLE)
		{
			//don't copy any death traps back to the segment
			if (mapCells[i].data.interactable != INTERACTABLE_TYPE_DEATH)
			{
				segmentMapArray[i] = INTERACTABLE_OFFSET + mapCells[i].data.interactable;
			}
			else
			{
				segmentMapArray[i] = EMPTY;
			}
		}
		else if (mapCells[i].type == CELL_TYPE_TILE && mapCells[i].data.tile == TILE_TYPE_BURNT_WALL)
		{
			//reset so called burnt wall to standard wall
			//using wallTile variable did not work for some reason;
			segmentMapArray[i] = TILE_TYPE_WALL_MIDDLE;
		}
	}
}

void addItemsToGroundLevel()
{
	//add a projectile weapon to the first segment of the first block
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_PROJECTILE_LAUNCHER, 0, -5);

	//add a different key to each of the 3 of 4 maze blocks
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_RED, 0, -2);
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_GREEN, 4, -2);
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_BLUE, 8, -2);

	//add a spanner to 1 maze block of 4 (16 in this case only works because there is only 1 item)
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_SPANNER, 0, -4);

	//add a medkit to 4 of 4 maze blocks
	addEntityToSegment(4, CELL_TYPE_ITEM, ITEM_TYPE_MEDKIT, 0, -2);

	//add a ammo to 2 segments of 4 maze blocks
	addEntityToSegment(8, CELL_TYPE_ITEM, ITEM_TYPE_PROJECTILE_AMMO, 0, -1);

	//add water pipes to 4 segments of 4 maze blocks
	addEntityToSegment(32, CELL_TYPE_ITEM, ITEM_TYPE_OBJECTIVE_WATER_PIPE, 0, 2);
}

void addItemsToMiddleLevel()
{
	//add a flamethrower weapon to the the first block
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_FLAMETHROWER, 0, -2);

	//add a different key to each of the 3 of 4 maze blocks
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_RED, 0, -2);
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_GREEN, 4, -2);
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_BLUE, 8, -2);

	//add a spanner to 1 maze block of 4 (16 in this case only works because there is only 1 item)
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_SPANNER, 0, -4);

	//add a medkit to 4 of 4 maze blocks
	addEntityToSegment(4, CELL_TYPE_ITEM, ITEM_TYPE_MEDKIT, 0, -2);

	//add a projectile ammo to 2 segments of 4 maze blocks
	addEntityToSegment(8, CELL_TYPE_ITEM, ITEM_TYPE_PROJECTILE_AMMO, 0, -1);

	//add flamethrower ammo to 4 segments of 4 maze blocks
	addEntityToSegment(16, CELL_TYPE_ITEM, ITEM_TYPE_FLAMETHROWER_AMMO, 0, 1);

	//add gears to 4 segments of 4 maze blocks
	addEntityToSegment(32, CELL_TYPE_ITEM, ITEM_TYPE_OBJECTIVE_GEAR, 0, 2);

	//add one test tube to 1 maze block of 4
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_TEST_TUBE, 0, -4);
}

void addItemsToTopLevel()
{
	//add a laser weapon to the the first block
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_LASER, 0, -2);

	//add a different key to each of the 3 of 4 maze blocks
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_RED, 0, -2);
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_GREEN, 4, -2);
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_KEY_BLUE, 8, -2);

	//add a spanner to 1 maze block of 4 (16 in this case only works because there is only 1 item)
	addEntityToSegment(1, CELL_TYPE_ITEM, ITEM_TYPE_SPANNER, 0, -4);

	//add a medkit to 4 of 4 maze blocks
	addEntityToSegment(4, CELL_TYPE_ITEM, ITEM_TYPE_MEDKIT, 0, -2);

	//add a projectile ammo to 2 segments of 4 maze blocks
	addEntityToSegment(4, CELL_TYPE_ITEM, ITEM_TYPE_PROJECTILE_AMMO, 0, -1);

	//add flamethrower ammo to 1 segments of 4 maze blocks
	addEntityToSegment(4, CELL_TYPE_ITEM, ITEM_TYPE_FLAMETHROWER_AMMO, 0, -2);

	//add a laser ammo to 1 segments of 4 maze blocks
	addEntityToSegment(4, CELL_TYPE_ITEM, ITEM_TYPE_LASER_AMMO, 0, -2);

	//add wirespools to 4 segments of 3 maze blocks
	addEntityToSegment(24, CELL_TYPE_ITEM, ITEM_TYPE_OBJECTIVE_WIRE_SPOOL, 0, 2);

	//add fuses to 4 segments of the last 2 maze blocks
	addEntityToSegment(24, CELL_TYPE_ITEM, ITEM_TYPE_OBJECTIVE_FUSE, 8, 3);
}

void addEntityToSegment(u8 itemCount, u8 type, u8 subType, u8 segmentStart, i8 entitiesPerSegment)
{
	/*
	1/1 we segment interval of 1
	1/2 we segment interval of 2 but choose randomly 1 of 2
	1/4 we segment interval of 4 but choose randomly 1 of 4
	1/16 we choose randomly 1 of 16
	*/
	
	u8 entitySegmentCount = entitiesPerSegment;
	u8 selectedSegment = segmentStart;

	//assume itemsPerSegment is positive for now
	//segmentInterval is what segment to jump to next
	u8 segmentInterval = 1;
	u8 offset = (type == CELL_TYPE_ITEM) ? ITEM_OFFSET : INTERACTABLE_OFFSET;

	while (itemCount)
	{
		//choose a random value for the next cell position
		u8 nextCell = cpct_rand() % 240;

		/*
		itemsPerSegment values
		1+ : an item in every segment
		-1 : an item in 1 out of 2 segments or 2/4 per block
		-2 : an item in 1 out of 4 segments or 1/4 per block
		-4 : an item in 1 out of 16 segments or 1 per level
		-5 : an item on the first segment of the level
		*/
		if (entitiesPerSegment == -1)
		{
			selectedSegment += cpct_rand() % 2;
			entitySegmentCount = 1;
			segmentInterval = 2;
		}
		else if (entitiesPerSegment == -2)
		{
			selectedSegment += cpct_rand() % 4;
			entitySegmentCount = 1;
			segmentInterval = 4;
		}
		else if (entitiesPerSegment == -4)
		{
			selectedSegment = cpct_rand() % 16;
			entitySegmentCount = 1;
		}
		else if (entitiesPerSegment == -5)
		{
			selectedSegment = 0;
			entitySegmentCount = 1;
		}

		//itemsPerSegment of 1 will always start at segmentStart

		while (entitySegmentCount)
		{
			//make sure we don't go out of range
			if (nextCell > 240)
			{
				nextCell = 0;
			}

			//make sure next cell is empty
			if (segments[segmentOrder[selectedSegment]][nextCell] == 255)
			{
				//add the an item to the empty cell
				segments[segmentOrder[selectedSegment]][nextCell] = offset + subType;

				//subtract the item from the count
				--itemCount;
				--entitySegmentCount;

				//choose a random value for the next cell position
				nextCell = cpct_rand() % 240;
			}
			else
			{
				//the cell is solid so we need to skip over to the next cell
				++nextCell;
			}
		}
		
		//drop off the lower bits that were added by the cpct_rand to clean the number for moving onto the next segment
		if (entitiesPerSegment == -1)
		{
			//interval of 2 we want to throw away just one
			selectedSegment &= ~1;
		}
		else if (entitiesPerSegment == -2)
		{
			//interval of 4 we want to throw away upto 3
			selectedSegment &= ~3;
		}

		//move onto the next segment based on the interval
		selectedSegment += segmentInterval;

		//reset item segment count
		entitySegmentCount = entitiesPerSegment;
	}
}

void drawEnemies()
{
    u8 i = 0;

	//the boss is only drawn on the last level in the bottom left segment
	if (currentLevel == TOP_LEVEL && currentSegment == BOSS_SEGMENT)
	{
		drawBoss();

		//enemies not drawn on the boss segment
		return;
	}

    for(i = 0; i<segmentEnemies[currentSegment]; ++i)
    {
        drawEnemy(&enemies[i]);
    }

	
}

void logicEnemies()
{
	u8 i = 0;
	
	if (currentLevel == TOP_LEVEL && currentSegment == BOSS_SEGMENT)
	{
		//enemies logic not required for boss segment
		return;
	}

	for (i = 0; i<segmentEnemies[currentSegment]; ++i)
	{
		logicEnemy(&enemies[i], (Actor*)&player, &player.collisionIndicator, &player.collisionFlickerCount, &player.inventory);
	}
}

void addDoor(u8 direction, u8* segment, u8 x, u8 y, u8 doorType)
{
	u8 exitPosition = x * SCREEN_TILES_Y + y;

	if (direction == DIRECTION_EAST || direction == DIRECTION_WEST)
	{
		//add the door
		segment[exitPosition] = INTERACTABLE_OFFSET + doorType;

		//add a second door (this prevents valves from blocking the door)
		segment[exitPosition+1] = INTERACTABLE_OFFSET + doorType;
	}
	else
	{
		//north or south

		//add the door
		segment[exitPosition] = INTERACTABLE_OFFSET + doorType;

		//add a second door (this prevents valves from blocking the door)
		segment[exitPosition + SCREEN_TILES_Y] = INTERACTABLE_OFFSET + doorType;
	}	
}

void addWaterBlocker()
{
	u8 i = 26;
	u8 selectedSegment = 0;
	u8 waterDistribution = 20;
	u8 waterOffset = TILE_TYPE_WATER_0;
	
	//add water across 1,2,3 blocks of the maze
	//(0 block is clear)

	//1 block is partially covered	
	for (selectedSegment = 4; selectedSegment < 8; ++selectedSegment)
	{
		while (i < 240)
		{
			//check it is empty cell
			if (segments[segmentOrder[selectedSegment]][i] == 255)
			{
				segments[segmentOrder[selectedSegment]][i] = waterOffset + (i % 4);
			}

			i += 1 + cpct_rand() % waterDistribution;
		}

		i = 0;
		waterDistribution -= 2;
	}

	i = 0;
	waterDistribution = 8;
	waterOffset = TILE_TYPE_WATER_4;
	
	//2 block is mostly covered
	for (selectedSegment = 8; selectedSegment < 12; ++selectedSegment)
	{
		while (i < 240)
		{
			//9 is where the entrance is so keeping it clear (maybe 8 could happen not sure though)
			if (selectedSegment == 8 || selectedSegment == 9)
			{
				if (i % SCREEN_TILES_Y == 0)
				{
					i += 1 + cpct_rand() % waterDistribution;
					continue;
				}
			}

			//check it is empty cell
			if (segments[segmentOrder[selectedSegment]][i] == 255)
			{
				segments[segmentOrder[selectedSegment]][i] = waterOffset + (i % 4);
			}

			i += 1 + cpct_rand() % waterDistribution;
		}

		i = 0;
		waterDistribution -= 2;
	}

	waterOffset = TILE_TYPE_WATER_8;

	//3 block is completely water covered except for the entrance	
	for (selectedSegment = 12; selectedSegment < 15; ++selectedSegment)
	{
		for (i = 0; i < 240; ++i)
		{
			//check it is empty cell
			if (segments[segmentOrder[selectedSegment]][i] == 255)
			{
				segments[segmentOrder[selectedSegment]][i] = waterOffset + (i % 4);
			}
		}
	}
	
	//assuming the exit is in the last square so we want the start of the end of the maze where the entrance is clear
	for (i = 0; i < 216; ++i)
	{
		//check it is empty cell
		if (segments[segmentOrder[15]][i] == 255)
		{
			segments[segmentOrder[15]][i] = waterOffset + (i % 4);
		}
	}
}

void addReleaseValves()
{
	//add a laser weapon to the the first block
	addEntityToSegment(VALVE_COUNT, CELL_TYPE_INTERACTABLE, INTERACTABLE_TYPE_VALVE, 0, 1);
}

void removeWaterBlocker()
{
	u8 i = 0;
	u8 segment = 0;

	//determines which block we are on
	//-1 because we start at water 0 and activatedValveCount starts at 1
	u8 firstSegment = ((activatedValveCount - 1) / 4) * 4;

	//chose the next water tile type 
	//-1 because we start at water 0 and activatedValveCount starts at 1
	u8 water = TILE_TYPE_WATER_0 + activatedValveCount - 1;

	//activatedValveCount=1 we remove TILE_TYPE_WATER_0
	//activatedValveCount=2 we remove TILE_TYPE_WATER_1
	//activatedValveCount=3 we remove TILE_TYPE_WATER_2
	//activatedValveCount=4 we remove TILE_TYPE_WATER_3

	//water starts from segment 4
	for (segment = 4 + firstSegment; segment < 8 + firstSegment; ++segment)
	{
		for (i = 0; i < 240; ++i)
		{
			if (segments[segmentOrder[segment]][i] == water)
			{
				segments[segmentOrder[segment]][i] = 255;
			}
		}
	}
}

void populateLevel(u8 level)
{
	if (level == GROUND_LEVEL)
	{
		u8 tankTypes[2] = { INTERACTABLE_OFFSET + INTERACTABLE_TYPE_WATER_TANK_0_EMPTY, INTERACTABLE_OFFSET + INTERACTABLE_TYPE_WATER_TANK_0_EMPTY };

		//add all the items to the ground level
		addItemsToGroundLevel();

		//add water tank interactables to ground level (before water)		
		addInteractableAndSpace(3, 3, 0, tankTypes, 2, HORIZONTAL_INTERACTABLE_, 8, 15, SEGMENT_TRANSITION_RANDOM_, 3, 3);

		//each tank is 2 and there are 8 interactables on the map
		totalTowerInteractableCount = 16;

		//ground level water
		addWaterBlocker();

		//ground level water release valves
		addReleaseValves();
	}
	else if (level == MIDDLE_LEVEL)
	{
		u8 generatorTypes[4] = { INTERACTABLE_OFFSET + INTERACTABLE_TYPE_GENERATOR_1_OFF, INTERACTABLE_OFFSET + INTERACTABLE_TYPE_GENERATOR_1_OFF, INTERACTABLE_OFFSET + INTERACTABLE_TYPE_GENERATOR_2_OFF,INTERACTABLE_OFFSET + INTERACTABLE_TYPE_GENERATOR_3_OFF };

		//middle level items
		addItemsToMiddleLevel();

		//middle level interactables (before infestation)
		//adds generator interactables for the middle level		
		addInteractableAndSpace(3, 3, 0, generatorTypes, 4, HORIZONTAL_INTERACTABLE_, 8, 13, SEGMENT_TRANSITION_RANDOM_, 5, 3);

		//each generator is 4 and there are 8 interactables on the map
		totalTowerInteractableCount = 32;

		//add infestation to the level (blocks 1,2,3)
		addInfestation();
	}
	else
	{
		//last level
		u8 transformerTypes[2] = { INTERACTABLE_OFFSET + INTERACTABLE_TYPE_TRANSFORMER_0_OFF, INTERACTABLE_OFFSET + INTERACTABLE_TYPE_TRANSFORMER_1_OFF };
		u8 teslaCoilTypes[2] = { INTERACTABLE_OFFSET + INTERACTABLE_TYPE_TESLA_COIL_0_OFF, INTERACTABLE_OFFSET + INTERACTABLE_TYPE_TESLA_COIL_1_OFF };

		//add the maximum space for the bottom left segment of the maze for where the boss will go
		addSpace(segments[BOSS_SEGMENT], SPACE_MAX_SIZE, 0, 0, 0, 0);

		//add transformers to the first 3 blocks of the level
		addInteractableAndSpace(3, 3, 0, transformerTypes, 2, HORIZONTAL_INTERACTABLE_, 12, 15, SEGMENT_TRANSITION_NEXT_, 3, 3);

		//add two lots of 4 tesla coils to the last block of the level
		addInteractableAndSpace(3, 3, 12, teslaCoilTypes, 2, VERTICIAL_INTERACTABLE_, 4, 6, SEGMENT_TRANSITION_NEXT_, 3, 4);
		addInteractableAndSpace(10, 3, 12, teslaCoilTypes, 2, VERTICIAL_INTERACTABLE_, 4, 7, SEGMENT_TRANSITION_NEXT_, 3, 4);			

		//12 transformers made up of 2, 8 telsa coils made up of 2
		totalTowerInteractableCount = 40;

		//last level items
		addItemsToTopLevel();

		//setup boss for last level
		initBoss(segments[BOSS_SEGMENT]);
	}
}

void addInteractableAndSpace(u8 xPositionStart, u8 yPositionStart, u8 startSegment, u8* interactableTypes, u8 typeCount, u8 typeDirection, u8 interactableCount, u8 xRandMax, u8 segmentTransition, u8 spaceWidth, u8 spaceHeight)
{
	u8 interactablePosition = (xPositionStart + 1) * SCREEN_TILES_Y + yPositionStart + 1;
	u8 nextSegment = startSegment;
	u8* nextSegmentPtr = NULL;
	u8 i = 0;
	u8 xPosition = 0;
	u8 yPosition = 0;

	//add transformers to first 3 blocks
	while (interactableCount)
	{
		nextSegmentPtr = segments[segmentOrder[nextSegment]];

		//randomly choose a position for the space
		xPosition = xPositionStart + cpct_rand() % xRandMax;
		yPosition = yPositionStart + cpct_rand() % 5;
		interactablePosition = (xPosition + 1) * SCREEN_TILES_Y + yPosition + 1;

		//create the space for the interactable
		addSpace(nextSegmentPtr, SPACE_FILLED_RECTANGLE, xPosition, yPosition, spaceWidth, spaceHeight);

		for (i = 0; i < typeCount; ++i)
		{
			if (typeDirection == HORIZONTAL_INTERACTABLE_)
			{
				//+= SCREEN_TILES_Y for interactables that are represented across multiple cells horizontally
				nextSegmentPtr[interactablePosition] = interactableTypes[i];
				interactablePosition += SCREEN_TILES_Y;
			}
			else
			{
				//++ for interactables that are represented across multiple cells vertically
				nextSegmentPtr[interactablePosition++] = interactableTypes[i];
			}
		}
		
		if (segmentTransition == SEGMENT_TRANSITION_RANDOM_)
		{
			//if nextSegment is even we need to add 2 for next segment
			//if nextSegment is odd we need to add 1 for next segment
			if ((nextSegment & 1) == 0)
			{
				nextSegment += 2;
			}
			else
			{
				++nextSegment;
			}

			//randomly choose a segment (next 1 of two segments in the block)
			nextSegment += cpct_rand() % 2;
		}
		else
		{
			//add the space to the next segment
			++nextSegment;
		}
			
		--interactableCount;
	}
}

void addInfestation()
{
	u8 i = 0;
	u8 selectedSegment = 0;

	//add infestation across 1,2,3 blocks of the maze

	//adding 1 blobs per segment for blocks 1,2
	for (selectedSegment = 4; selectedSegment < 12; ++selectedSegment)
	{
		//add the blob somewhere in the segment making sure not to block off any exits
		u8 xPosition = 3 + cpct_rand() % 13;
		u8 yPosition = 1 + cpct_rand() % 6;
		addInfestationBlob(xPosition, yPosition, selectedSegment);
	}

	//adding 2 blobs per segment for blocks 3
	for (selectedSegment = 12; selectedSegment < 16; ++selectedSegment)
	{
		//left side blob
		u8 xPosition = 2 + cpct_rand() % 4;
		u8 yPosition = 1 + cpct_rand() % 6;		
		addInfestationBlob(xPosition, yPosition, selectedSegment);
		
		//right side blob
		xPosition = 8 + cpct_rand() % 8;
		yPosition = 1 + cpct_rand() % 6;
		addInfestationBlob(xPosition, yPosition, selectedSegment);
	}

	//add horizontal infestation line/vine to block 2
	for (selectedSegment = 8; selectedSegment < 12; ++selectedSegment)
	{
		//horizontal
		u8 yPosition = 0;
		u8 xPosition = 0;

		u8 startPosition = 5 + cpct_rand() % 3;
		i = startPosition;

		while (i <= 240)
		{
			//this prevents infestation from being added to the top row
			if (i % SCREEN_TILES_Y == 0)
			{
				++i;
			}

			//this prevents infestation from beingadded to the bottom row
			if (i % SCREEN_TILES_Y == 11)
			{
				--i;
			}

			//check it is empty cell
			if (segments[segmentOrder[selectedSegment]][i] == EMPTY)
			{
				segments[segmentOrder[selectedSegment]][i] = TILE_TYPE_EMPTY_MIDDLE_INFESTED;
				++infestedCellCount;
			}
			else if (segments[segmentOrder[selectedSegment]][i] == TILE_TYPE_WALL_TEST)
			{
				segments[segmentOrder[selectedSegment]][i] = TILE_TYPE_WALL_MIDDLE_INFESTED;
				++infestedCellCount;
			}

			i += 11 + cpct_rand() % 3;		
		}

		i = 0;
	}	

	//vertical into diagonal line/vine to block 3
	for (selectedSegment = 12; selectedSegment < 16; ++selectedSegment)
	{
		u8 yPosition = 0;
		u8 xPosition = 3 + cpct_rand() % 3;

		u8 startPosition = (xPosition) * SCREEN_TILES_Y + yPosition;
		i = startPosition;

		//240-12 so nothing blocks the first column into the level from the left
		while (i <= 228)
		{
			//check it is empty cell
			if (segments[segmentOrder[selectedSegment]][i] == 255)
			{
				segments[segmentOrder[selectedSegment]][i] = TILE_TYPE_EMPTY_MIDDLE_INFESTED;
				++infestedCellCount;
			}
			else if (segments[segmentOrder[selectedSegment]][i] == TILE_TYPE_WALL_TEST)
			{
				segments[segmentOrder[selectedSegment]][i] = TILE_TYPE_WALL_MIDDLE_INFESTED;
				++infestedCellCount;
			}

			//this is to prevent two infested blocks being placed next to each other on the top of the column
			if (i % SCREEN_TILES_Y == 0)
			{
				++i;
			}
			else if (cpct_rand() % 2 == 1)
			{
				//next cell on x
				i += SCREEN_TILES_Y;
			}
			else
			{
				//next cell on y
				++i;
			}

			//this prevents infestation from being placed at the bottom tile row of the level potentially blocking access to the segment
			if (i % SCREEN_TILES_Y == 11)
			{
				++i;
			}
		}

		i = 0;
	}

	//save the total infested cell count for objective dialog
	totalInfestedCellCount = infestedCellCount;
}

void addInfestationBlob(u8 xPosition, u8 yPosition, u8 selectedSegment)
{
	i8 startOffset = 1;
	u8 currentColumn = 0;

	u8 startPosition = (xPosition) * SCREEN_TILES_Y + yPosition;
	u8 height = 2; //height doesnt include start position
	u8 cellCount = 16;
	u8 endColumn = 0;
	u8 i = startPosition + startOffset;
	endColumn = i + height;

	while (cellCount)
	{
		//check it is empty cell
		if (segments[segmentOrder[selectedSegment]][i] == EMPTY)
		{
			segments[segmentOrder[selectedSegment]][i] = TILE_TYPE_EMPTY_MIDDLE_INFESTED;
			++infestedCellCount;
		}
		else if (segments[segmentOrder[selectedSegment]][i] == wallTile)
		{
			segments[segmentOrder[selectedSegment]][i] = TILE_TYPE_WALL_MIDDLE_INFESTED;
			++infestedCellCount;
		}

		if (i < endColumn)
		{
			++i;
		}
		else
		{
			++currentColumn;

			if (currentColumn == 3)
			{
				startOffset = -1;
			}

			i += SCREEN_TILES_Y - height - startOffset;
			height = 4;
			endColumn = i + height;
			startOffset = 0;
		}

		--cellCount;
	}
}

void distributeEnemies()
{
	u8 i = 0;
	u8 x = 0;
	u8 y = 0;
	u8 enemyType = 0;

	for (i = 0; i<segmentEnemies[currentSegment]; ++i)
	{
		x = 3 + cpct_rand() % 14;
		y = 3 + cpct_rand() % 7;

		while(MAPA(x, y).type != CELL_TYPE_EMPTY)
		{
			if (++y >= 10)
			{
				y = 3;
			}

			if (++x >= 17)
			{
				x = 3;
			}
		}
		
		if(enemyDifficulty == EASY_DIFFICULTY)
		{
			enemyType = ENEMY_TYPE_ROBOT;
		}
		else if (enemyDifficulty == HARD_DIFFICULTY)
		{
			enemyType = ENEMY_TYPE_CHASER;
		}
		else
		{
			if (i % 2 == 0)
			{
				enemyType = ENEMY_TYPE_ROBOT;
			}
			else
			{
				enemyType = ENEMY_TYPE_CHASER;
			}
		}

		initEnemy(&enemies[i], i, x, y, enemyType);		
	}
}

void setupDeathTraps()
{
	/*
	u8 i = 0;
	u8 x = 0;
	u8 y = 0;
	u8 position = 0;

	//distribute deathtraps around the screen
	for (i = 0; i < DEATH_TRAP_MAX_COUNT_; ++i)
	{
		x = 3 + cpct_rand() % 14;
		y = 3 + cpct_rand() % 7;
		
		while (1)
		{
			position = XY_TO_POSITION(x, y);

			if (mapCells[position].type == CELL_TYPE_EMPTY);
			{
				break;
			}

			if (++y >= 10)
			{
				y = 3;
			}

			if (++x >= 17)
			{
				x = 3;
			}
		}

		//update segment with deathtrap information
		mapCells[position].type = CELL_TYPE_INTERACTABLE;
		mapCells[position].data.interactable = INTERACTABLE_TYPE_DEATH_TRAP_WHIRLPOOL;

		//initalise the death trap values
		deathTraps[i].cellPosition = position;
		deathTraps[i].xPosition = x;
		deathTraps[i].yPosition = y;
		deathTraps[i].timer = gameTimer + 75 * cpct_rand() % 9;
	}*/
}

void logicDeathTraps()
{
	/*
	u8 i = 0;

	for (i = 0; i < DEATH_TRAP_MAX_COUNT_; ++i)
	{
		logicDeathTrap(&deathTraps[i]);
	}*/
}

