#ifndef MAP_H
#define MAP_H

#include <cpctelera.h>
#include "Boss.h"
#include "Cell.h"
#include "DeathTrap.h"
#include "GameState.h"
#include "GameTypes.h"
#include "Enemy.h"
#include "EnemyState.h"
#include "Maze.h"
#include "Object.h"
#include "Player.h"
#include "Space.h"
#include "sprites/TileSprites.h"
#include "sprites/ItemSprites.h"
#include "Tile.h"

#define SEGMENT_COUNT 16
#define SEGMENTS_X 4
#define SEGMENTS_Y 4

//used for accessing the map array throughout the code
extern u8* mapArray;

//determine when a map redraw should happen
extern u8 mapRedraw;

//the segments that make up the level
extern u8* segments[SEGMENT_COUNT];

void initMap(u8 seed);

void drawMap();

//converts an array to a double size array but in a tall form so we can address each room easily
void bigMapTallCombined(u8* oneArray, u8* twoArray, u8 rowLength);

//transition the screen to a new segment
void moveSegment(u8 direction, Object* playerObject);

//draws the enemies on the map
void drawEnemies();

void logicEnemies();

//add a door interactable to the exit of a particular maze block segment
void addDoor(u8 direction, u8* segment, u8 x, u8 y, u8 doorType);

//updates a segment based on which cells are now empty/changed in the cell grid
void copyCellChangesToSegment(u8 segment);

//adding items to levels
void addItemsToGroundLevel();
void addItemsToMiddleLevel();
void addItemsToTopLevel();

//addss all the water to 12 segments on the ground level
void addWaterBlocker();

//removes the next segment of water from ground level
void removeWaterBlocker();

//water release valves for ground level
void addReleaseValves();

//adds items or interactables to segments of the level
void addEntityToSegment(u8 itemCount, u8 type, u8 subType, u8 segmentStart, i8 itemsPerSegment);

//populates the map for the specified level
void populateLevel(u8 level);

//adds interactables and spaces to the level segments
void addInteractableAndSpace(u8 xPositionStart, u8 yPositionStart, u8 startSegment, u8* interactableTypes, u8 typeCount, u8 typeDirection, u8 interactableCount, u8 xRandMax, u8 segmentTransition, u8 spaceWidth, u8 spaceHeight);

//adds the infestation for the middle level
void addInfestation();

//adds a blob of infestation to the chosen segment
void addInfestationBlob(u8 xPosition, u8 yPosition, u8 selectedSegment);

//adds enemies to the map segment
void distributeEnemies();

//setup deathtraps for the specified segment (intended to be before cell init)
void setupDeathTraps();

#endif