#include "Menu.h"

#define MENU_ITEM_COUNT 4
//5
u8 currentMenuItem_;

const u8* COLOUR_MODE_MENU_TEXT_;
const u8 GREEN_MODE_TEXT_[12] = "green mode ";
const u8 COLOUR_MODE_TEXT_[12] = "colour mode";

const u8* DIFFICULTY_MENU_TEXT_;
const u8 EASY_DIFFICULTY_TEXT_[18] = "easy difficulty  ";
const u8 NORMAL_DIFFICULTY_TEXT_[18] = "normal difficulty";
const u8 HARD_DIFFICULTY_TEXT_[18] = "hard difficulty  ";

u8 chosenDifficulty_;

#define SCORE_COUNT 5
#define MAX_NAME_LENGTH 10

u16 highScores[SCORE_COUNT];
u8 highScoreNames[SCORE_COUNT][11];

#define START_GAME_ 0
#define REBIND_CONTROLS_ 1
#define DIFFICULTY_ 2
#define HIGH_SCORES_ 3
#define COLOUR_MODE_ 4

void runMenu()
{
    //setup game palette for ground level
    choosePalette(GROUND_LEVEL);
    
    //initalise high scores
    initScores();

	//setup the default keys for the player
	setupDefaultKeys();

    //default mode is colour mode
    COLOUR_MODE_MENU_TEXT_ = COLOUR_MODE_TEXT_;

	//default difficulty is normal difficulty
	chosenDifficulty_ = NORMAL_DIFFICULTY;	
	DIFFICULTY_MENU_TEXT_ = NORMAL_DIFFICULTY_TEXT_;

    //draw the menu initially
    drawMenu();

	//initial choose the first menu item
	currentMenuItem_ = 0;

    while(1)
    {
        if(cpct_isKeyPressed(Key_Return))
        {
            switch(currentMenuItem_)
            {
                case START_GAME_:
                {	
					//set difficulty
					setDifficulty(chosenDifficulty_);
						
					//runs the full game session
					runGameSession();

					//display the end screen
					displayEndScreen();

                    //update and show high scores
                    updateHighScores();
                }
                break;

                case REBIND_CONTROLS_:
                    rebindControls();
                break;

                case HIGH_SCORES_:
					displayHighScores();	
                break;

                case DIFFICULTY_:
					chooseDifficulty();
                break;

				//dont need this now
                /*case COLOUR_MODE_:
                    colourMode();
                break;*/

            }

            //clear the screen after the previous option ends
            cpct_clearScreen(0);

            //redraw the menu following the previous option ends
            drawMenu();

            //wait for enter not to be pressed any more
            while(cpct_isKeyPressed(Key_Return))
            {                
            }
        }
        else if(cpct_isKeyPressed(Key_CursorUp) || cpct_isKeyPressed(Joy0_Up))
        {
            //0 is the minimum
            if(currentMenuItem_)
            {
                --currentMenuItem_;

				//update indicator position
				drawIndicator();
            }

			//wait for up not to be pressed any more
            while(cpct_isKeyPressed(Key_CursorUp) || cpct_isKeyPressed(Joy0_Up))
            {
            }
        }
        else if(cpct_isKeyPressed(Key_CursorDown) || cpct_isKeyPressed(Joy0_Down))
        {
            //item count cant reach max menu item count
            if(currentMenuItem_ + 1 < MENU_ITEM_COUNT)
            {
                ++currentMenuItem_;

				//update indicator position
				drawIndicator();
            }
            
			//wait for down not to be pressed any more
            while(cpct_isKeyPressed(Key_CursorDown) || cpct_isKeyPressed(Joy0_Down))
            {
            }
        }
    }
}

void drawMenu()
{
    //title
    drawString(10,5, "fitzroy and the infestation");

    //actual menu options
    drawString(25, 45, "start game");
    drawString(25, 60, "rebind controls");
    drawString(25, 75, DIFFICULTY_MENU_TEXT_);
    drawString(25, 90, "high scores");    
    //drawString(25, 105, COLOUR_MODE_MENU_TEXT_);
    
    //the dot indicating which menu option is selected
	drawIndicator();
}

void drawIndicator()
{
	u8 x = 20;
	u8 y = 45;

	//draw over where the indicator can go
	u8* memory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);
	
	cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0), 2, 15 * MENU_ITEM_COUNT);

	//draw the current position of the indicator
	y = 45 + currentMenuItem_ * 15;
	memory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);
	cpct_drawSolidBox(memory, cpct_px2byteM0(12, 12), 2, 8);
}

void rebindControls()
{
    cpct_clearScreen(0);

    //title
    drawString(15,8, "enter controls");

    //draw the text, make sure no key is currently down then set the key for the next control
    drawString(25,30, "up");
    gameControls.up = getKey();

    drawString(25,45, "down");
    gameControls.down = getKey();

    drawString(25,60, "left");
    gameControls.left = getKey();

    drawString(25,75, "right");
    gameControls.right = getKey();

    drawString(25,90, "shoot");
    gameControls.shoot = getKey();

	drawString(25,105, "weapon");
	gameControls.weapon = getKey();

    drawString(25,120, "exit");
    gameControls.exit = getKey();

	drawString(25,135, "objectives");
    gameControls.objective = getKey();
}

cpct_keyID getKey() 
{
    // Traverse the keystatus vector from end to start, to help
    // the compiler better optimise this code
    u8 i = 10;
    u8* keys = cpct_keyboardStatusBuffer + 9;
    u16 keypressed;

    // wait for any previous key presses to clear first
    while (cpct_isAnyKeyPressed());

    //then wait for the desired key press
    while (!cpct_isAnyKeyPressed());

    
    // Analyse which key has been pressed and return its keycode
    do 
    {
        // We analyse groups of keys byte by byte (8 by 8, 1 bit per key)
        // If no single key is pressed in this group, all bits will be on, and the byte will be 0xFF
        // Then, XORing the byte with 0xFF will result in 0 unless some key from this group is pressed
        keypressed = *keys ^ 0xFF;

		if (keypressed)
		{
			return (keypressed << 8) + (i - 1); // Convert to cpct_keyID format: 8 first bits = key mask, 8 next bits, keyboard row (0-9)
		}

        --keys;
    }
    while(--i);

    // No key was pressed 
    return 0;
}

void colourMode()
{
	//update menu item text to reflect the correct mode
	if (currentPalette == GREEN_PALETTE)
	{
		choosePalette(GROUND_PALETTE);
		COLOUR_MODE_MENU_TEXT_ = COLOUR_MODE_TEXT_;
	}
	else
	{
		choosePalette(GREEN_PALETTE);
		COLOUR_MODE_MENU_TEXT_ = GREEN_MODE_TEXT_;
	}
}

void initScores()
{
    u8 i = 0;

    for(i = 0; i<SCORE_COUNT; ++i)
    {
        highScores[i] = 100;
        cpct_memcpy(highScoreNames[i], "cpc\0", 4);
    }
}

void displayHighScores()
{
    u8 i = 0;

    cpct_clearScreen(0);

    //title
    drawString(25, 5, "high scores");

    //entries
    for(i = 0; i<SCORE_COUNT; ++i)
    {
		u8 output[6];
		u8 y = 45 + i * 15;
        drawString(40, y, highScoreNames[i]);
		
		convertIntToChar(highScores[i], output);
		drawString(25, y, output);
    }

	getKey();
}

void updateHighScores()
{
	//check that the player got a higher score than the bottom score
	if (gameScore > highScores[4])
	{
		//enter name
		u8 letterCount = 0;
		u8 scoreName[11];
        u8 highScoreText[6];
		u8 i = 0;

		//default to anything
		cpct_keyID chosenKey = Key_0;
				
		//set scoreName to the null terminator
		cpct_memset(scoreName, '\0', 11);
		
		cpct_clearScreen(0);
		drawString(15, 5, "new high score");
        convertIntToChar(gameScore, highScoreText);
        drawString(45, 5, highScoreText);
        drawString(20, 25, "enter your name");

		do
		{
			//get the next character from the user
			chosenKey = getKey();

			//del to move back to the previous key
			if (chosenKey == Key_Del || chosenKey == 16386) //16386 is wincpctelera specific
			{
				//go back to the previous letter
				if (letterCount)
				{
					--letterCount;

					scoreName[letterCount] = ' ';

					//draw letter
					drawString(25 + letterCount * 2, 50, &scoreName[letterCount]);
				}
			}
			//make sure the key is in the correct range and the word is not too long
			else if (letterCount < MAX_NAME_LENGTH)
			{
				u8 chosenCharacter = 0;

				switch (chosenKey)
				{
					case Key_A:
						chosenCharacter = 'a';
					break;

					case Key_B:
						chosenCharacter = 'b';
					break;

					case Key_C:
						chosenCharacter = 'c';
						break;

					case Key_D:
						chosenCharacter = 'd';
						break;

					case Key_E:
						chosenCharacter = 'e';
					break;

					case Key_F:
						chosenCharacter = 'f';
					break;

					case Key_G:
						chosenCharacter = 'g';
					break;

					case Key_H:
						chosenCharacter = 'h';
					break;

					case Key_I:
						chosenCharacter = 'i';
					break;

					case Key_J:
						chosenCharacter = 'j';
					break;

					case Key_K:
						chosenCharacter = 'k';
					break;

					case Key_L:
						chosenCharacter = 'l';
					break;

					case Key_M:
						chosenCharacter = 'm';
					break;

					case Key_N:
						chosenCharacter = 'n';
					break;

					case Key_O:
						chosenCharacter = 'o';
					break;

					case Key_P:
						chosenCharacter = 'p';
					break;

					case Key_Q:
						chosenCharacter = 'q';
					break;

					case Key_R:
						chosenCharacter = 'r';
					break;

					case Key_S:
						chosenCharacter = 's';
					break;

					case Key_T:
						chosenCharacter = 't';
					break;

					case Key_U:
						chosenCharacter = 'u';
					break;

					case Key_V:
						chosenCharacter = 'v';
					break;

					case Key_W:
						chosenCharacter = 'w';
					break;

					case Key_X:
						chosenCharacter = 'x';
					break;

					case Key_Y:
						chosenCharacter = 'y';
					break;

					case Key_Z:
						chosenCharacter = 'z';
					break;

						//space does not work on wincpctelera
					case Key_Space:
						chosenCharacter = ' ';
					break;
				}

				if (chosenCharacter)
				{
					scoreName[letterCount] = chosenCharacter;

					//draw letter
					drawString(25 + letterCount * 2, 50, &scoreName[letterCount]);

					++letterCount;
				}
			}
		}
		while (chosenKey != Key_Return && chosenKey != Key_Enter);

		//update score values
		highScores[4] = gameScore;

		for (i = 4; i > 0; --i)
		{
			if (highScores[i] > highScores[i - 1])
			{
				highScores[i] = highScores[i - 1];
				cpct_memcpy(highScoreNames[i], highScoreNames[i - 1], 11);

				highScores[i - 1] = gameScore;
				cpct_memcpy(highScoreNames[i - 1], scoreName, 11);
			}
		}
	}
}

void chooseDifficulty()
{
	//switch between difficulties
	if(chosenDifficulty_ == EASY_DIFFICULTY)
	{
		chosenDifficulty_ = NORMAL_DIFFICULTY;
		DIFFICULTY_MENU_TEXT_ = NORMAL_DIFFICULTY_TEXT_;
	}
	else if(chosenDifficulty_ == NORMAL_DIFFICULTY)
	{
		chosenDifficulty_ = HARD_DIFFICULTY;
		DIFFICULTY_MENU_TEXT_ = HARD_DIFFICULTY_TEXT_;
	}
	else
	{
		chosenDifficulty_ = EASY_DIFFICULTY;
		DIFFICULTY_MENU_TEXT_ = EASY_DIFFICULTY_TEXT_;
	}
}

void displayEndScreen()
{
	// wait for any previous key presses to clear first
	while (cpct_isAnyKeyPressed());

	//clear the screen
	cpct_clearScreen(0);

	if (currentLevel == LEVEL_COUNT)
	{		
		u8* memory;
		u8 count = 255;
		u8 colour = 0;
		u8 x = 0;
		u8 y = 0;

		//patterned background
		while (count)
		{
			colour = 1 + cpct_rand() % 15;
			y = 14 * (cpct_rand() % 14);
			x = 4 * (cpct_rand() % 17);

			memory = cpct_getScreenPtr(CPCT_VMEM_START, x, y);
			cpct_drawSolidBox(memory, cpct_px2byteM0(colour, colour), 10, 12);

			--count;
		}

		//victorious label
		memory = cpct_getScreenPtr(CPCT_VMEM_START, 25, 60);
		cpct_drawSolidBox(memory, cpct_px2byteM0(0, 0), 30, 20);
		drawString(30, 65, "victorious");

		//1000 points for winning the game
		gameScore += 1000;
	}
	else
	{
		drawString(30, 20, "game over");
	}

	//reset palette to ground palette after the game has finished
	choosePalette(GROUND_PALETTE);

	//then wait for the desired key press
	while (!cpct_isAnyKeyPressed());
}

void loadingScreen()
{
	//clear the screen so something is happening
	cpct_clearScreen(0);

	drawString(30, 65, "loading");
}