#ifndef PLAYER_H
#define PLAYER_H

#include <cpctelera.h>
#include "Actor.h"
#include "Cell.h"
#include "Dialog.h"
#include "Difficulty.h"
#include "GameState.h"
#include "Hud.h"
#include "Interactable.h"
#include "Item.h"
#include "Keys.h"
#include "Map.h"
#include "Object.h"
#include "Objective.h"
#include "Projectile.h"
#include "sprites/PlayerSprites.h"
#include "sprites/TileSprites.h"
#include "Timer.h"

#define PLAYER_WIDTH 4
#define PLAYER_HEIGHT 16

#define PROJECTILE_WEAPON 1
#define FLAMETHROWER_WEAPON 2
#define LASER_WEAPON 3

#define PLAYER_MAX_HEALTH 10

#define INVENTORY_ITEM_KEY 1
#define INVENTORY_ITEM_SPANNER 2
#define INVENTORY_ITEM_KEY_GREEN 4
#define INVENTORY_ITEM_KEY_BLUE 8
#define INVENTORY_ITEM_TEST_TUBE 16

typedef struct Player Player;

struct Player
{
	//actor properties
    Object object;
    u8 backgroundSprite[64];
	u16 timer;
	u8 health;

	//player properties
    Projectile projectile;
	u8 inventory;
	u8 direction : 2;
    u8 projectileAmmo : 6; //8	
	u8 currentWeapon : 2;
    u8 flamethrowerAmmo : 6; //8
	u8 hasProjectileLauncher : 1; 	
    u8 hasFlamethrower : 1;
    u8 laserAmmo : 6; //8    
	u8 collisionIndicator;
	u8 collisionFlickerCount;
	u8 hasLaser : 1;
	u8 lives : 3;
};

//global player object
extern Player player;

extern u16 weaponSwitchTimer;

//initialises the player object values
void initPlayer();

//draws the player on the screen as positioned in the player object
void drawPlayer();

//moves the player position based on keyboard input
void movePlayer();

//runs any logic required for the player function such as interactions and velocity/position
void logicPlayer();

u8 processCell(u8 x, u8 y);

//fire/shoot the current weapon the player is holding if they have a weapon
void playerShoot();

//stops shooting the current weapon if it is a laser or flamethrower
void playerEndShoot();

//determines the first empty position where the player can be placed on the map
void determinePlayerPosition(u8* x, u8* y);

#endif