#ifndef PROJECTILE_H
#define PROJECTILE_H

#include <cpctelera.h>
#include "Boss.h"
#include "Cell.h"
#include "DrawQueue.h"
#include "EnemyState.h"
#include "GameState.h"
#include "GameTypes.h"
#include "Object.h"
#include "sprites/PlayerSprites.h"

#define BULLET_PROJECTILE_WIDTH 2
#define BULLET_PROJECTILE_HEIGHT 4
#define FLAME_PRJECTILE_VERTICAL_WIDTH 3
#define FLAME_PRJECTILE_VERTICAL_HEIGHT 24
#define FLAME_PRJECTILE_HORIZONTAL_WIDTH 6
#define FLAME_PRJECTILE_HORIZONTAL_HEIGHT 12
#define BULLET_PROJECTILE 1
#define FLAME_PROJECTILE 2
#define LASER_PROJECTILE 3

typedef struct Projectile Projectile;

struct Projectile
{
    Object object;

    //graphics memory for previous location
    u8* backgroundMemory;

    //0 inactive, bullet, flame, laserbeam
    u8 type: 2;

	//direction that the projectile is travelling/pointing towards
	u8 direction : 2;

	//for flaging when the flamethrower is used to destroy infestation
	u8 infestationFlag : 1;

	//for indicating that a projectile has been fired by a projectile launcher
	u8 projectileActive : 1;
};

void initProjectile(Projectile* projectile, u8 type, u8 x, u8 y, u8 direction, u8 playerWidth, u8 playerHeight, u8* graphicsMemory);

void drawProjectile(Projectile* projectile);

void drawPreviousBackgroundMemory(Projectile* projectile);
void drawProjectileBackgroundMemoryAtPosition(Projectile* projectile);

void logicProjectile(Projectile* projectile, Object* parent);

//copy the background at the projectile current position
void copyProjectileBackground(Projectile* projectile);
void copyPreviousProjectileBackground(Projectile* projectile);

void destroyBulletProjectile(Projectile* projectile);

#endif